@@ -1,6 +1,6 @@
 /************************************************************************
  *
- * $Id: geos_c.cpp 2251 2009-01-20 01:01:24Z pramsey $
+ * $Id: geos_c.cpp 2742 2009-11-26 18:29:56Z warmerdam $
  *
  * C-Wrapper for GEOS library
  *
@@ -16,6 +16,7 @@
  ***********************************************************************/
 
 #include <geos/geom/prep/PreparedGeometryFactory.h> 
+#include <geos/index/strtree/STRtree.h>
 #include <geos/io/WKTReader.h>
 #include <geos/io/WKBReader.h>
 #include <geos/io/WKTWriter.h>
@@ -23,11 +24,17 @@
 #include <geos/io/CLocalizer.h>
 #include <geos/operation/overlay/OverlayOp.h>
 #include <geos/operation/union/CascadedPolygonUnion.h>
+#include <geos/algorithm/distance/DiscreteHausdorffDistance.h>
+
+#ifdef _MSC_VER
+#pragma warning(disable : 4099)
+#endif
 
 // Some extra magic to make type declarations in geos_c.h work - for cross-checking of types in header.
 #define GEOSGeometry geos::geom::Geometry
 #define GEOSPreparedGeometry geos::geom::prep::PreparedGeometry
 #define GEOSCoordSequence geos::geom::CoordinateSequence
+#define GEOSSTRtree geos::index::strtree::STRtree
 #define GEOSWKTReader_t geos::io::WKTReader
 #define GEOSWKTWriter_t geos::io::WKTWriter
 #define GEOSWKBReader_t geos::io::WKBReader
@@ -40,11 +47,15 @@
 ///
 #undef VERBOSE_EXCEPTIONS
 
+#include <geos/export.h>
+
+/*
 #if defined(_MSC_VER)
 #  define GEOS_DLL     __declspec(dllexport)
 #else
 #  define GEOS_DLL
 #endif
+*/
 
 // import the most frequently used definitions globally
 using geos::geom::Geometry;
@@ -59,11 +70,12 @@ using geos::io::WKBReader;
 using geos::io::WKBWriter;
 using geos::io::CLocalizer;
 
+using geos::index::strtree::STRtree;
+
 using geos::operation::overlay::OverlayOp;
 using geos::operation::overlay::overlayOp;
 using geos::operation::geounion::CascadedPolygonUnion;
 
-
 typedef std::auto_ptr<Geometry> GeomAutoPtr;
 
 //## GLOBALS ################################################
@@ -82,9 +94,18 @@ initGEOS (GEOSMessageHandler nf, GEOSMessageHandler ef)
 void
 finishGEOS ()
 {
-    finishGEOS_r( handle );
+    if (handle != NULL) {
+        finishGEOS_r( handle );
+        handle = NULL;
+    }
 }
 
+void 
+GEOSFree (void* buffer) 
+{ 
+    GEOSFree_r( handle, buffer ); 
+} 
+    
 /****************************************************************
 ** relate()-related functions
 ** return 0 = false, 1 = true, 2 = error occured
@@ -194,6 +215,18 @@ GEOSDistance(const Geometry *g1, const Geometry *g2, double *dist)
     return GEOSDistance_r( handle, g1, g2, dist );
 }
 
+int
+GEOSHausdorffDistance(const Geometry *g1, const Geometry *g2, double *dist)
+{
+    return GEOSHausdorffDistance_r( handle, g1, g2, dist );
+}
+
+int
+GEOSHausdorffDistanceDensify(const Geometry *g1, const Geometry *g2, double densifyFrac, double *dist)
+{
+    return GEOSHausdorffDistanceDensify_r( handle, g1, g2, densifyFrac, dist );
+}
+
 int
 GEOSArea(const Geometry *g, double *area)
 {
@@ -304,6 +337,22 @@ GEOSBuffer(const Geometry *g1, double width, int quadrantsegments)
     return GEOSBuffer_r( handle, g1, width, quadrantsegments );
 }
 
+Geometry *
+GEOSBufferWithStyle(const Geometry *g1, double width, int quadsegs,
+	int endCapStyle, int joinStyle, double mitreLimit)
+{
+    return GEOSBufferWithStyle_r( handle, g1, width, quadsegs, endCapStyle,
+                               joinStyle, mitreLimit );
+}
+
+Geometry *
+GEOSSingleSidedBuffer(const Geometry *g1, double width, int quadsegs,
+	int joinStyle, double mitreLimit, int leftSide)
+{
+    return GEOSSingleSidedBuffer_r( handle, g1, width, quadsegs, 
+                               joinStyle, mitreLimit, leftSide );
+}
+
 Geometry *
 GEOSConvexHull(const Geometry *g1)
 {
@@ -455,9 +504,15 @@ GEOSLineMerge(const Geometry *g)
 }
 
 int
-GEOSGetSRID(const Geometry *g1)
+GEOSGetSRID(const Geometry *g)
 {
-    return GEOSGetSRID_r( handle, g1 );
+    return GEOSGetSRID_r( handle, g );
+}
+
+void
+GEOSSetSRID(Geometry *g, int srid)
+{
+    return GEOSSetSRID_r( handle, g, srid );
 }
 
 char 
@@ -704,14 +759,14 @@ GEOSWKBWriter_destroy(WKBWriter *Writer)
 }
 
 
-/* The owner owns the result */
+/* The caller owns the result */
 unsigned char*
 GEOSWKBWriter_write(WKBWriter *writer, const Geometry *geom, size_t *size)
 {
     return GEOSWKBWriter_write_r( handle, writer, geom, size );
 }
 
-/* The owner owns the result */
+/* The caller owns the result */
 unsigned char*
 GEOSWKBWriter_writeHEX(WKBWriter *writer, const Geometry *geom, size_t *size)
 {
@@ -795,4 +850,78 @@ GEOSPreparedIntersects(const geos::geom::prep::PreparedGeometry *pg1, const Geom
     return GEOSPreparedIntersects_r( handle, pg1, g2 );
 }
 
+STRtree *
+GEOSSTRtree_create (size_t nodeCapacity)
+{
+    return GEOSSTRtree_create_r( handle, nodeCapacity );
+}
+
+void
+GEOSSTRtree_insert (geos::index::strtree::STRtree *tree,
+                    const geos::geom::Geometry *g,
+                    void *item)
+{
+    GEOSSTRtree_insert_r( handle, tree, g, item );
+}
+
+void
+GEOSSTRtree_query (geos::index::strtree::STRtree *tree,
+                   const geos::geom::Geometry *g, 
+                   GEOSQueryCallback cb,
+                   void *userdata)
+{
+    GEOSSTRtree_query_r( handle, tree, g, cb, userdata );
+}
+
+void 
+GEOSSTRtree_iterate(geos::index::strtree::STRtree *tree,
+                    GEOSQueryCallback callback,
+                    void *userdata)
+{
+    GEOSSTRtree_iterate_r( handle, tree, callback, userdata );
+}
+
+char
+GEOSSTRtree_remove (geos::index::strtree::STRtree *tree,
+                    const geos::geom::Geometry *g,
+                    void *item)
+{
+    return GEOSSTRtree_remove_r( handle, tree, g, item );
+}
+
+void
+GEOSSTRtree_destroy (geos::index::strtree::STRtree *tree)
+{
+    GEOSSTRtree_destroy_r( handle, tree );
+}
+
+double
+GEOSProject (const geos::geom::Geometry *g,
+             const geos::geom::Geometry *p)
+{
+    return GEOSProject_r (handle, g, p);
+}
+
+geos::geom::Geometry *
+GEOSInterpolate (const geos::geom::Geometry *g,
+                 double d)
+{
+    return GEOSInterpolate_r(handle, g, d);
+}
+
+double
+GEOSProjectNormalized (const geos::geom::Geometry *g,
+                       const geos::geom::Geometry *p)
+{
+    return GEOSProjectNormalized_r (handle, g, p);
+}
+
+geos::geom::Geometry *
+GEOSInterpolateNormalized (const geos::geom::Geometry *g,
+                           double d)
+{
+    return GEOSInterpolateNormalized_r(handle, g, d);
+}
+
+
 } /* extern "C" */
