@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: MCPointInRing.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: MCPointInRing.cpp 2477 2009-05-06 17:10:01Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -36,15 +36,18 @@ namespace algorithm { // geos.algorithm
 
 MCPointInRing::MCSelecter::MCSelecter(const Coordinate& newP,
 		MCPointInRing *prt)
+	:
+	MonotoneChainSelectAction()
 {
 	p=newP;
 	parent=prt;
 }
 
+/* public overridden */
 void
-MCPointInRing::MCSelecter::select(LineSegment *ls)
+MCPointInRing::MCSelecter::select(const LineSegment& ls)
 {
-	parent->testLineSegment(p,ls);
+	parent->testLineSegment(p, ls);
 }
 
 MCPointInRing::MCPointInRing(const LinearRing *newRing)
@@ -72,14 +75,29 @@ MCPointInRing::buildIndex()
 //	Envelope *env=ring->getEnvelopeInternal();
 	tree=new bintree::Bintree();
 	pts=CoordinateSequence::removeRepeatedPoints(ring->getCoordinatesRO());
-	vector<chain::MonotoneChain*> *mcList=chain::MonotoneChainBuilder::getChains(pts);
-	for(int i=0;i<(int)mcList->size();i++) {
+
+	// NOTE: we take ownership of mcList and it's elements
+	vector<chain::MonotoneChain*> *mcList =
+		chain::MonotoneChainBuilder::getChains(pts);
+
+	for(size_t i=0, n=mcList->size(); i<n; ++i)
+	{
 		chain::MonotoneChain *mc=(*mcList)[i];
-		Envelope *mcEnv=mc->getEnvelope();
-		interval.min=mcEnv->getMinY();
-		interval.max=mcEnv->getMaxY();
-		tree->insert(&interval,mc);
+		const Envelope& mcEnv = mc->getEnvelope();
+		interval.min = mcEnv.getMinY();
+		interval.max = mcEnv.getMaxY();
+
+		// TODO: is 'mc' ownership transferred here ? (see below)
+		//       by documentation SpatialIndex does NOT take
+		//       ownership of the items, so unless we query it
+		//       all later we've a leak problem here..
+		//       Need a focused testcase.
+		//
+		tree->insert(&interval, mc);
 	}
+
+	// TODO: mcList elements ownership went to tree or what ?
+
 	delete mcList;
 }
 
@@ -123,7 +141,7 @@ MCPointInRing::testMonotoneChain(Envelope *rayEnv,
 }
 
 void
-MCPointInRing::testLineSegment(Coordinate& p,LineSegment *seg)
+MCPointInRing::testLineSegment(const Coordinate& p, const LineSegment& seg)
 {
 	double xInt;  // x intersection of segment with ray
 	double x1;    // translated coordinates
@@ -132,23 +150,26 @@ MCPointInRing::testLineSegment(Coordinate& p,LineSegment *seg)
 	double y2;
 
 	/*
-	*  Test if segment crosses ray from test point in positive x direction.
-	*/
-	Coordinate& p1=seg->p0;
-	Coordinate& p2=seg->p1;
-	x1=p1.x-p.x;
-	y1=p1.y-p.y;
-	x2=p2.x-p.x;
-	y2=p2.y-p.y;
-	if (((y1>0)&&(y2<=0)) || ((y2>0)&&(y1<=0))) {
+	 * Test if segment crosses ray from test point in positive x direction.
+	 */
+	const Coordinate& p1 = seg.p0;
+	const Coordinate& p2 = seg.p1;
+	x1 = p1.x - p.x;
+	y1 = p1.y - p.y;
+	x2 = p2.x - p.x;
+	y2 = p2.y - p.y;
+
+	if (((y1>0)&&(y2<=0)) || ((y2>0)&&(y1<=0)))
+	{
+
 		/*
-		*  segment straddles x axis, so compute intersection.
-		*/
+		 *  segment straddles x axis, so compute intersection.
+		 */
 		xInt=RobustDeterminant::signOfDet2x2(x1,y1,x2,y2)/(y2-y1);
-		//xsave=xInt;
+
 		/*
-		*  crosses ray if strictly positive intersection.
-		*/
+		 *  crosses ray if strictly positive intersection.
+		 */
 		if (0.0<xInt) {
 			crossings++;
 		}
