@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: GeometryFactory.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: GeometryFactory.cpp 2617 2009-08-02 23:15:54Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -75,7 +75,7 @@ GeometryFactory::GeometryFactory()
 {
 #if GEOS_DEBUG
 	std::cerr << "GEOS_DEBUG: GeometryFactory["<<this<<"]::GeometryFactory()" << std::endl;
-	std::cerr << " \ create PrecisionModel["<<precisionModel<<"]" << std::endl;
+	std::cerr << "\tcreate PrecisionModel["<<precisionModel<<"]" << std::endl;
 #endif
 }
 
@@ -232,7 +232,7 @@ GeometryFactory::createPoint() const
 Point*
 GeometryFactory::createPoint(const Coordinate& coordinate) const
 {
-	if (coordinate==Coordinate::getNull()) {
+	if (coordinate.isNull()) {
 		return createPoint();
 	} else {
 		CoordinateSequence *cl=coordinateListFactory->create(new vector<Coordinate>(1, coordinate));
