@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: LineString.cpp 2091 2007-12-21 22:49:03Z mloskot $
+ * $Id: LineString.cpp 2441 2009-04-30 12:07:26Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -12,14 +12,18 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: geom/LineString.java rev. 1.46
+ *
  **********************************************************************/
 
 #include <geos/util/IllegalArgumentException.h> 
 #include <geos/algorithm/CGAlgorithms.h>
-#include <geos/operation/IsSimpleOp.h>
 #include <geos/geom/Coordinate.h>
 #include <geos/geom/CoordinateSequenceFactory.h>
 #include <geos/geom/CoordinateSequence.h>
+#include <geos/geom/CoordinateSequenceFilter.h>
 #include <geos/geom/CoordinateFilter.h>
 #include <geos/geom/Dimension.h>
 #include <geos/geom/GeometryFilter.h>
@@ -50,7 +54,7 @@ LineString::LineString(const LineString &ls)
 	//points=ls.points->clone();
 }
 
-LineString*
+Geometry*
 LineString::reverse() const
 {
 	assert(points.get());
@@ -203,19 +207,15 @@ LineString::getGeometryType() const
 	return "LineString";
 }
 
-bool
-LineString::isSimple() const
-{
-	operation::IsSimpleOp iso;
-	return iso.isSimple(this); 
-}
-
 Geometry*
 LineString::getBoundary() const
 {
 	if (isEmpty()) {
-		return getFactory()->createEmptyGeometry();
+		return getFactory()->createMultiPoint();
 	}
+
+	// using the default OGC_SFS MOD2 rule, the boundary of a
+	// closed LineString is empty
 	if (isClosed()) {
 		return getFactory()->createMultiPoint();
 	}
@@ -382,6 +382,32 @@ LineString::apply_ro(GeometryComponentFilter *filter) const
 	filter->filter_ro(this);
 }
 
+void
+LineString::apply_rw(CoordinateSequenceFilter& filter)
+{
+	size_t npts=points->size();
+	if (!npts) return;
+	for (size_t i = 0; i<npts; ++i)
+	{
+		filter.filter_rw(*points, i);
+		if (filter.isDone()) break;
+	}
+	if (filter.isGeometryChanged()) geometryChanged();
+}
+
+void
+LineString::apply_ro(CoordinateSequenceFilter& filter) const
+{
+	size_t npts=points->size();
+	if (!npts) return;
+	for (size_t i = 0; i<npts; ++i)
+	{
+		filter.filter_ro(*points, i);
+		if (filter.isDone()) break;
+	}
+	//if (filter.isGeometryChanged()) geometryChanged();
+}
+
 GeometryTypeId
 LineString::getGeometryTypeId() const
 {
