@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: MultiPolygon.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: MultiPolygon.cpp 2437 2009-04-30 11:29:05Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -11,6 +11,10 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: geom/MultiPolygon.java rev 1.34
+ *
  **********************************************************************/
 
 #include <geos/geom/Geometry.h>
@@ -60,7 +64,7 @@ bool MultiPolygon::isSimple() const {
 
 Geometry* MultiPolygon::getBoundary() const {
 	if (isEmpty()) {
-		return getFactory()->createGeometryCollection(NULL);
+		return getFactory()->createMultiLineString();
 	}
 	vector<Geometry *>* allRings=new vector<Geometry *>();
 	for (size_t i = 0; i < geometries->size(); i++) {
