@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: Polygon.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: Polygon.cpp 2678 2009-10-17 12:28:41Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -14,7 +14,7 @@
  *
  **********************************************************************
  *
- * Last port: geom/Polygon.java rev. 1.47
+ * Last port: geom/Polygon.java rev. 1.50
  *
  **********************************************************************/
 
@@ -29,6 +29,7 @@
 #include <geos/geom/Envelope.h>
 #include <geos/geom/CoordinateSequenceFactory.h>
 #include <geos/geom/CoordinateSequence.h>
+#include <geos/geom/CoordinateSequenceFilter.h>
 #include <geos/geom/GeometryFilter.h>
 #include <geos/geom/GeometryComponentFilter.h>
 
@@ -73,7 +74,7 @@ Polygon::Polygon(LinearRing *newShell, vector<Geometry *> *newHoles,
 	}
 	else
 	{
-		if (newShell->isEmpty() && hasNonEmptyElements(newHoles)) {
+		if (newHoles != NULL && newShell->isEmpty() && hasNonEmptyElements(newHoles)) {
 			delete newShell;
 			delete newHoles;
 			throw util::IllegalArgumentException("shell is empty but holes are not");
@@ -106,30 +107,21 @@ Polygon::getCoordinates() const
 		return getFactory()->getCoordinateSequenceFactory()->create(NULL);
 	}
 
-	size_t i, j, npts;
-	size_t nholes=holes->size();
-
 	vector<Coordinate> *cl = new vector<Coordinate>;
 
+	// reserve space in the vector for all the polygon points
+	cl->reserve(getNumPoints());
+
 	// Add shell points
 	const CoordinateSequence* shellCoords=shell->getCoordinatesRO();
-	npts=shellCoords->getSize();
+	shellCoords->toVector(*cl);
 
-	/*
-	 * reserve space in the vector as if all holes have the same
-	 * amount of points. Holes usually have less, so this should
-	 * be a good compromise
-	 */
-	cl->reserve((nholes+1)*npts);
-
-	for (j=0; j<npts; ++j) cl->push_back(shellCoords->getAt(j));
-
-	for (i=0; i<nholes; ++i)
+	// Add holes points
+	size_t nholes=holes->size();
+	for (size_t i=0; i<nholes; ++i)
 	{
-		// Add hole points
 		const CoordinateSequence* childCoords=((LinearRing *)(*holes)[i])->getCoordinatesRO();
-		npts=childCoords->getSize();
-		for (j=0; j<npts; ++j) cl->push_back(childCoords->getAt(j));
+		childCoords->toVector(*cl);
 	}
 
 	return getFactory()->getCoordinateSequenceFactory()->create(cl);
@@ -208,7 +200,7 @@ Polygon::getBoundary() const
 	const GeometryFactory* gf = getFactory();
 
 	if (isEmpty()) {
-		return gf->createEmptyGeometry();
+		return gf->createMultiLineString();
 	}
 
 	if ( ! holes->size() )
@@ -409,6 +401,40 @@ Polygon::apply_rw(GeometryComponentFilter *filter)
 	}
 }
 
+void
+Polygon::apply_rw(CoordinateSequenceFilter& filter)
+{
+	shell->apply_rw(filter);
+
+	if (! filter.isDone())
+	{
+		for (size_t i=0, n=holes->size(); i<n; ++i)
+		{
+			(*holes)[i]->apply_rw(filter);
+			if (filter.isDone())
+			break;
+        	}
+	}
+	if (filter.isGeometryChanged()) geometryChanged();
+}
+
+void
+Polygon::apply_ro(CoordinateSequenceFilter& filter) const
+{
+	shell->apply_ro(filter);
+
+	if (! filter.isDone())
+	{
+		for (size_t i=0, n=holes->size(); i<n; ++i)
+		{
+			(*holes)[i]->apply_ro(filter);
+			if (filter.isDone())
+			break;
+        	}
+	}
+	//if (filter.isGeometryChanged()) geometryChanged();
+}
+
 Polygon::~Polygon()
 {
 	delete shell;
