@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: GeometryTransformer.cpp 2127 2008-05-20 21:25:21Z mloskot $
+ * $Id: GeometryTransformer.cpp 2757 2009-12-01 15:39:41Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -115,7 +115,7 @@ GeometryTransformer::transformCoordinates(
 		const Geometry* parent)
 {
 
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformCoordinates(CoordinateSequence " << coords <<", Geometry " << parent << ");" << std::endl;
 #endif
@@ -128,7 +128,7 @@ GeometryTransformer::transformPoint(
 		const Point* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformPoint(Point " << geom <<", Geometry " << parent << ");" << std::endl;
@@ -145,7 +145,7 @@ GeometryTransformer::transformMultiPoint(
 		const MultiPoint* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformMultiPoint(MultiPoint " << geom <<", Geometry " << parent << ");" << std::endl;
@@ -176,7 +176,7 @@ GeometryTransformer::transformLinearRing(
 		const LinearRing* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformLinearRing(LinearRing " << geom <<", Geometry " << parent << ");" << std::endl;
@@ -202,7 +202,7 @@ GeometryTransformer::transformLineString(
 		const LineString* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformLineString(LineString " << geom <<", Geometry " << parent << ");" << std::endl;
@@ -218,7 +218,7 @@ GeometryTransformer::transformMultiLineString(
 		const MultiLineString* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformMultiLineString(MultiLineString " << geom <<", Geometry " << parent << ");" << std::endl;
@@ -249,7 +249,7 @@ GeometryTransformer::transformPolygon(
 		const Polygon* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformPolygon(Polygon " << geom <<", Geometry " << parent << ");" << std::endl;
@@ -322,13 +322,13 @@ GeometryTransformer::transformMultiPolygon(
 		const MultiPolygon* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformMultiPolygon(MultiPolygon " << geom <<", Geometry " << parent << ");" << std::endl;
 #endif
 
-	vector<Geometry*>* transGeomList = new vector<Geometry*>();
+	auto_ptr< vector<Geometry*> > transGeomList( new vector<Geometry*>() );
 
 	for (unsigned int i=0, n=geom->getNumGeometries(); i<n; i++)
 	{
@@ -344,7 +344,7 @@ GeometryTransformer::transformMultiPolygon(
 		transGeomList->push_back(transformGeom.release());
 	}
 
-	return Geometry::AutoPtr(factory->buildGeometry(transGeomList));
+	return Geometry::AutoPtr(factory->buildGeometry(transGeomList.release()));
 
 }
 
@@ -353,7 +353,7 @@ GeometryTransformer::transformGeometryCollection(
 		const GeometryCollection* geom,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 #if GEOS_DEBUG
 	std::cerr << "GeometryTransformer::transformGeometryCollection(GeometryCollection " << geom <<", Geometry " << parent << ");" << std::endl;
