@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: MCIndexNoder.h 2240 2009-01-18 20:34:09Z pramsey $
+ * $Id: MCIndexNoder.h 2778 2009-12-03 19:44:00Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -11,11 +11,17 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: noding/MCIndexNoder.java rev. 1.6 (JTS-1.9)
+ *
  **********************************************************************/
 
 #ifndef GEOS_NODING_MCINDEXNODER_H
 #define GEOS_NODING_MCINDEXNODER_H
 
+#include <geos/export.h>
+
 #include <geos/inline.h>
 
 #include <geos/index/chain/MonotoneChainOverlapAction.h> // for inheritance
@@ -50,7 +56,7 @@ namespace noding { // geos.noding
  *
  * Last port: noding/MCIndexNoder.java rev. 1.4 (JTS-1.7)
  */
-class MCIndexNoder : public SinglePassNoder {
+class GEOS_DLL MCIndexNoder : public SinglePassNoder {
 
 private:
 	std::vector<index::chain::MonotoneChain*> monoChains;
@@ -86,23 +92,21 @@ public:
 	void computeNodes(std::vector<SegmentString*>* inputSegmentStrings);
 
 	class SegmentOverlapAction : public index::chain::MonotoneChainOverlapAction {
-	private:
-		SegmentIntersector& si;
 	public:
 		SegmentOverlapAction(SegmentIntersector& newSi)
 			:
+			index::chain::MonotoneChainOverlapAction(),
 			si(newSi)
 		{}
 
-		void overlap(index::chain::MonotoneChain* mc1, int start1,
-				index::chain::MonotoneChain* mc2, int start2);
+		void overlap(index::chain::MonotoneChain& mc1, size_t start1,
+            index::chain::MonotoneChain& mc2, size_t start2);
+    private:
+        SegmentIntersector& si;
 
-		void overlap(geom::LineSegment* s1, geom::LineSegment* s2)
-        {
-            UNREFERENCED_PARAMETER(s1);
-            UNREFERENCED_PARAMETER(s2);
-            assert(0);
-        }
+        // Declare type as noncopyable
+        SegmentOverlapAction(const SegmentOverlapAction& other);
+        SegmentOverlapAction& operator=(const SegmentOverlapAction& rhs);
 	};
 	
 };
