@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: MonotoneChainBuilder.cpp 2194 2008-09-23 23:01:00Z mloskot $
+ * $Id: MonotoneChainBuilder.cpp 2538 2009-06-04 15:01:08Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -11,6 +11,10 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: index/chain/MonotoneChainBuilder.java rev 1.12 (JTS-1.10)
+ *
  **********************************************************************/
 
 #include <geos/index/chain/MonotoneChainBuilder.h> 
@@ -18,6 +22,7 @@
 #include <geos/geom/CoordinateSequence.h>
 #include <geos/geomgraph/Quadrant.h>
 
+#include <cassert>
 #include <cstdio>
 #include <vector>
 
@@ -51,77 +56,87 @@ void
 MonotoneChainBuilder::getChains(const CoordinateSequence* pts, void* context,
                                 vector<MonotoneChain*>& mcList)
 {
-	vector<int> startIndex;
-	getChainStartIndices(pts, startIndex);
+	vector<size_t> startIndex;
+	getChainStartIndices(*pts, startIndex);
 	size_t nindexes = startIndex.size();
 	if (nindexes > 0)
 	{
 		size_t n = nindexes - 1;
 		for(size_t i = 0; i < n; i++)
 		{
-			MonotoneChain* mc = new MonotoneChain(pts, startIndex[i], startIndex[i+1], context);
+			MonotoneChain* mc = new MonotoneChain(*pts, startIndex[i], startIndex[i+1], context);
 			mcList.push_back(mc);
 		}
 	}
 }
 
-/**
- * Return an array containing lists of start/end indexes of the monotone chains
- * for the given list of coordinates.
- * The last entry in the array points to the end point of the point array,
- * for use as a sentinel.
- */
+/* static public */
 void
-MonotoneChainBuilder::getChainStartIndices(const CoordinateSequence *pts,
-                                           vector<int>& startIndexList)
+MonotoneChainBuilder::getChainStartIndices(const CoordinateSequence& pts,
+                                           vector<size_t>& startIndexList)
 {
 	// find the startpoint (and endpoints) of all monotone chains
 	// in this edge
-	int start = 0;
+	size_t start = 0;
 	startIndexList.push_back(start);
-    const std::size_t n = pts->getSize() - 1;
+	const size_t n = pts.getSize() - 1;
 	do
-    {
-		int last = findChainEnd(pts, start);
+	{
+		size_t last = findChainEnd(pts, start);
 		startIndexList.push_back(last);
 		start = last;
-	} while (static_cast<std::size_t>(start) < n);
+	} while (start < n);
 
 }
 
-/**
- * @return the index of the last point in the monotone chain starting
- * at <code>start</code>.
- */
-int
-MonotoneChainBuilder::findChainEnd(const CoordinateSequence *pts, int start)
+/* private static */
+size_t
+MonotoneChainBuilder::findChainEnd(const CoordinateSequence& pts, size_t start)
 {
-    std::size_t npts = pts->getSize();
 
-	// skip any zero-length segments at start of sequence
-	while ( pts->getAt( start).equals2D( pts->getAt( start + 1) ) )
-		if ( ++start >= ( npts - 1 ) )
-			// bail if there are no non-zero-length segments
-			return npts - 1;
-			
-	// determine quadrant for chain
-	int chainQuad=Quadrant::quadrant(pts->getAt(start),pts->getAt(start + 1));
-	std::size_t last=start+1;
+	const size_t npts = pts.getSize(); // cache
+
+	assert(start < npts);
+	assert(npts); // should be implied by the assertion above,
+	              // 'start' being unsigned
+
+	size_t safeStart = start;
+
+        // skip any zero-length segments at the start of the sequence
+        // (since they cannot be used to establish a quadrant)
+	while ( safeStart < npts - 1
+		&& pts[safeStart].equals2D(pts[safeStart+1]) ) 
+	{
+		++safeStart;
+	}
+
+	// check if there are NO non-zero-length segments
+	if (safeStart >= npts - 1) {
+		return npts - 1;
+	}
+
+	// determine overall quadrant for chain
+	// (which is the starting quadrant)
+	int chainQuad = Quadrant::quadrant(pts[safeStart],
+	                                   pts[safeStart + 1]);
+	size_t last = start + 1;
 	while (last < npts)
 	{
-		// skip a zero-length segnments
-		if (! pts->getAt( last - 1).equals2D( pts->getAt( last) ) )
+		// skip zero-length segments, but include them in the chain
+		if (! pts[last - 1].equals2D( pts[last] ) )
 		{
 			// compute quadrant for next possible segment in chain
-			int quad=Quadrant::quadrant(pts->getAt(last-1),pts->getAt(last));
-			if (quad!=chainQuad) break;
+			int quad = Quadrant::quadrant( pts[last - 1],
+			                               pts[last]      );
+			if (quad != chainQuad) break;
 		}
-		last++;	
+		++last;	
 	}
 #if GEOS_DEBUG
 	std::cerr<<"MonotoneChainBuilder::findChainEnd() returning"<<std::endl;
 #endif
-	return static_cast<int>(last - 1);
+
+	return last - 1;
 }
 
 } // namespace geos.index.chain
