@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: WKTReader.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: WKTReader.cpp 2754 2009-11-30 13:55:12Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -12,11 +12,16 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: io/WKTReader.java rev. 1.1 (JTS-1.7)
+ *
  **********************************************************************/
 
 #include <geos/io/WKTReader.h>
 #include <geos/io/StringTokenizer.h>
 #include <geos/io/ParseException.h>
+#include <geos/io/CLocalizer.h>
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/Coordinate.h>
 #include <geos/geom/Point.h>
@@ -57,6 +62,7 @@ Geometry *
 WKTReader::read(const string &wellKnownText)
 {
 	//auto_ptr<StringTokenizer> tokenizer(new StringTokenizer(wellKnownText));
+        CLocalizer clocale;
 	StringTokenizer tokenizer(wellKnownText);
 	Geometry *g=NULL;
 	g=readGeometryTaggedText(&tokenizer);
@@ -326,7 +332,6 @@ WKTReader::readMultiPointText(StringTokenizer *tokenizer)
 		}
 		err << endl;
 		throw ParseException(err.str());
-		return NULL;
 	}
 }
 
