@@ -1,9 +1,10 @@
 /**********************************************************************
- * $Id: BufferBuilder.cpp 2231 2009-01-05 23:42:08Z swongu $
+ * $Id: BufferBuilder.cpp 2658 2009-10-05 19:29:45Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
  *
+ * Copyright (C) 2009 Sandro Santilli <strk@keybit.net>
  * Copyright (C) 2005-2007 Refractions Research Inc.
  * Copyright (C) 2001-2002 Vivid Solutions Inc.
  *
@@ -14,32 +15,39 @@
  *
  **********************************************************************
  *
- * Last port: operation/buffer/BufferBuilder.java rev. 1.23 (JTS-1.7)
+ * Last port: operation/buffer/BufferBuilder.java rev. 1.30 (JTS-1.10)
  *
  **********************************************************************/
 
-#include <geos/operation/overlay/PolygonBuilder.h> 
-#include <geos/operation/overlay/OverlayNodeFactory.h> 
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/Location.h>
 #include <geos/geom/Geometry.h>
 #include <geos/geom/Polygon.h>
 #include <geos/geom/GeometryCollection.h>
+#include <geos/geom/LineString.h>
+#include <geos/geom/MultiLineString.h>
 #include <geos/operation/buffer/BufferBuilder.h>
 #include <geos/operation/buffer/OffsetCurveBuilder.h>
 #include <geos/operation/buffer/OffsetCurveSetBuilder.h>
 #include <geos/operation/buffer/BufferSubgraph.h>
 #include <geos/operation/buffer/SubgraphDepthLocater.h>
+#include <geos/operation/overlay/OverlayOp.h>
+#include <geos/operation/overlay/snap/SnapOverlayOp.h> 
+#include <geos/operation/overlay/PolygonBuilder.h> 
+#include <geos/operation/overlay/OverlayNodeFactory.h> 
+#include <geos/operation/linemerge/LineMerger.h>
 #include <geos/algorithm/LineIntersector.h>
 #include <geos/noding/IntersectionAdder.h>
 #include <geos/noding/SegmentString.h>
 #include <geos/noding/MCIndexNoder.h>
+#include <geos/noding/NodedSegmentString.h>
 #include <geos/geomgraph/Position.h>
 #include <geos/geomgraph/PlanarGraph.h>
 #include <geos/geomgraph/Label.h>
 #include <geos/geomgraph/Node.h>
 #include <geos/geomgraph/Edge.h>
 #include <geos/util/GEOSException.h>
+#include <geos/util/IllegalArgumentException.h>
 #include <geos/profiler.h>
 
 #include <cassert>
@@ -48,6 +56,9 @@
 #include <algorithm>
 #include <iostream>
 
+// Debug single sided buffer
+//#define GEOS_DEBUG_SSB 1
+
 #ifndef GEOS_DEBUG
 #define GEOS_DEBUG 0
 #endif
@@ -62,6 +73,7 @@ using namespace geos::geomgraph;
 using namespace geos::noding;
 using namespace geos::algorithm;
 using namespace geos::operation::overlay;
+using namespace geos::operation::linemerge;
 
 namespace geos {
 namespace operation { // geos.operation
@@ -98,6 +110,220 @@ BufferBuilder::~BufferBuilder()
 		delete newLabels[i];
 }
 
+/*public*/
+Geometry*
+BufferBuilder::bufferLineSingleSided( const Geometry* g, double distance,
+                                      bool leftSide )
+{
+   // Returns the line used to create a single-sided buffer.
+   // Input requirement: Must be a LineString.
+   const LineString* l = dynamic_cast< const LineString* >( g );
+   if ( !l ) throw util::IllegalArgumentException("BufferBuilder::bufferLineSingleSided only accept linestrings");
+
+   // Get geometry factory and precision model.
+   const PrecisionModel* precisionModel = workingPrecisionModel;
+   if ( !precisionModel ) precisionModel = l->getPrecisionModel();
+
+   assert( precisionModel );
+   assert( l );
+
+   geomFact = l->getFactory();
+
+   // First, generate the two-sided buffer using a butt-cap.
+   BufferParameters modParams = bufParams;
+   modParams.setEndCapStyle(BufferParameters::CAP_FLAT); 
+   Geometry* buf = 0;
+
+   // This is a (temp?) hack to workaround the fact that
+   // BufferBuilder BufferParamaters are immutable after
+   // construction, while we want to force the end cap
+   // style to FLAT for single-sided buffering
+   {
+      BufferBuilder tmp(modParams);
+      buf = tmp.buffer( l, distance );
+   }
+
+   // Create MultiLineStrings from this polygon.
+   Geometry* bufLineString = buf->getBoundary();
+
+#ifdef GEOS_DEBUG_SSB
+   std::cerr << "input|" << *l << std::endl;
+   std::cerr << "buffer|" << *bufLineString << std::endl;
+#endif
+
+   // Then, get the raw (i.e. unnoded) single sided offset curve.
+   OffsetCurveBuilder curveBuilder( precisionModel, modParams );
+   std::vector< CoordinateSequence* > lineList;
+
+   std::auto_ptr< CoordinateSequence > coords ( g->getCoordinates() );
+   curveBuilder.getSingleSidedLineCurve( coords.get(), distance,
+                                         lineList, leftSide, !leftSide );
+   coords.reset();
+
+   // Create a SegmentString from these coordinates.
+   SegmentString::NonConstVect curveList;
+   for ( unsigned int i = 0; i < lineList.size(); ++i )
+   {
+      CoordinateSequence* seq = lineList[i];
+
+      SegmentString* ss = new NodedSegmentString(seq, NULL);
+      curveList.push_back( ss );
+   }
+
+   // Node these SegmentStrings.
+   Noder* noder = getNoder( precisionModel );
+   noder->computeNodes( &curveList );
+   SegmentString::NonConstVect* nodedEdges = noder->getNodedSubstrings();
+
+   // Create a geometry out of the noded substrings.
+   std::vector< Geometry* >* singleSidedNodedEdges =
+      new std::vector< Geometry * >();
+   for ( unsigned int i = 0, n = nodedEdges->size(); i < n; ++i )
+   {
+      SegmentString* ss = ( *nodedEdges )[i];
+
+      Geometry* tmp = geomFact->createLineString(
+                        ss->getCoordinates()->clone()
+                      );
+      singleSidedNodedEdges->push_back( tmp );
+   }
+
+   if ( nodedEdges != &curveList ) delete nodedEdges;
+
+   for (size_t i=0, n=curveList.size(); i<n; ++i) delete curveList[i];
+   curveList.clear();
+
+   for (size_t i=0, n=lineList.size(); i<n; ++i) delete lineList[i];
+   lineList.clear();
+
+
+   Geometry* singleSided = geomFact->createMultiLineString(
+      singleSidedNodedEdges );
+
+#ifdef GEOS_DEBUG_SSB
+     std::cerr << "edges|" << *singleSided << std::endl;
+#endif
+
+   // Use the boolean operation intersect to obtain the line segments lying
+   // on both the butt-cap buffer and this multi-line.
+   //Geometry* intersectedLines = singleSided->intersection( bufLineString );
+   // NOTE: we use Snapped overlay because the actual buffer boundary might
+   //       diverge from original offset curves due to the addition of
+   //       intersections with caps and joins curves
+   using geos::operation::overlay::snap::SnapOverlayOp;
+   Geometry* intersectedLines = SnapOverlayOp::overlayOp(*singleSided, *bufLineString, OverlayOp::opINTERSECTION).release();
+
+#ifdef GEOS_DEBUG_SSB
+     std::cerr << "intersection" << "|" << *intersectedLines << std::endl;
+#endif
+
+   // Merge result lines together.
+   LineMerger lineMerge;
+   lineMerge.add( intersectedLines );
+   std::auto_ptr< std::vector< LineString* > > mergedLines (
+	lineMerge.getMergedLineStrings() );
+
+   // Convert the result into a std::vector< Geometry* >.
+   std::vector< Geometry* >* mergedLinesGeom = new std::vector< Geometry* >();
+   const Coordinate& startPoint = l->getCoordinatesRO()->front();
+   const Coordinate& endPoint = l->getCoordinatesRO()->back();
+   while( !mergedLines->empty() )
+   {
+      // Remove end points if they are a part of the original line to be
+      // buffered.
+      CoordinateSequence::AutoPtr coords(mergedLines->back()->getCoordinates());
+      if ( NULL != coords.get() )
+      {
+         // Use 98% of the buffer width as the point-distance requirement - this
+         // is to ensure that the point that is "distance" +/- epsilon is not
+         // included.
+         const double ptDistAllowance = 0.98 * distance;
+         // Use 102% of the buffer width as the line-length requirement - this
+         // is to ensure that line segments that is length "distance" +/-
+         // epsilon is removed.
+         const double segLengthAllowance = 1.02 * distance;
+
+         // Clean up the front of the list.
+         // Loop until the line's end is not inside the buffer width from
+         // the startPoint.
+         while ( coords->size() > 1 && 
+                 coords->front().distance( startPoint ) < ptDistAllowance )
+         {
+            // Record the end segment length.
+            double segLength = coords->front().distance( ( *coords )[1] );
+            // Stop looping if there are no more points, or if the segment
+            // length is larger than the buffer width.
+            if ( coords->size() <= 1 || segLength > segLengthAllowance )
+            {
+               break;
+            }
+            // If the first point is less than buffer width away from the
+            // reference point, then delete the point.
+            coords->deleteAt( 0 );
+         }
+         while ( coords->size() > 1 && 
+                 coords->front().distance( endPoint ) < ptDistAllowance )
+         {
+            double segLength = coords->front().distance( ( *coords )[1] );
+            if ( coords->size() <= 1 || segLength > segLengthAllowance )
+            {
+               break;
+            }
+            coords->deleteAt( 0 );
+         }
+
+         // Clean up the back of the list.
+         while ( coords->size() > 1 &&
+                 coords->back().distance( startPoint ) < ptDistAllowance )
+         {
+            double segLength = coords->back().distance(
+               ( *coords )[coords->size()-2] );
+            if ( coords->size() <= 1 || segLength > segLengthAllowance )
+            {
+               break;
+            }
+            coords->deleteAt( coords->size()-1 );
+         }
+         while ( coords->size() > 1 &&
+                 coords->back().distance( endPoint ) < ptDistAllowance )
+         {
+            double segLength = coords->back().distance(
+               ( *coords )[coords->size()-2] );
+            if ( coords->size() <= 1 || segLength > segLengthAllowance )
+            {
+               break;
+            }
+            coords->deleteAt( coords->size()-1 );
+         }
+
+         // Add the coordinates to the resultant line string.
+         if ( coords->size() > 1 )
+         {
+            mergedLinesGeom->push_back( geomFact->createLineString( coords.release() ) );
+         }
+      }
+
+      geomFact->destroyGeometry( mergedLines->back() );
+      mergedLines->pop_back();
+   }
+
+   // Clean up.
+   if ( noder != workingNoder ) delete noder;
+   geomFact->destroyGeometry( buf );
+   geomFact->destroyGeometry( bufLineString );
+   geomFact->destroyGeometry( singleSided );
+   geomFact->destroyGeometry( intersectedLines );
+
+   if ( mergedLinesGeom->size() > 1 ) return geomFact->createMultiLineString( mergedLinesGeom );
+   else
+   {
+      // Must be a single line
+      Geometry* single = (*mergedLinesGeom)[0];
+      delete mergedLinesGeom;
+      return single;
+   }
+}
+
 /*public*/
 Geometry*
 BufferBuilder::buffer(const Geometry *g, double distance)
@@ -113,8 +339,7 @@ BufferBuilder::buffer(const Geometry *g, double distance)
 	// factory must be the same as the one used by the input
 	geomFact=g->getFactory();
 
-	OffsetCurveBuilder curveBuilder(precisionModel, quadrantSegments);
-	curveBuilder.setEndCapStyle(endCapStyle);
+	OffsetCurveBuilder curveBuilder(precisionModel, bufParams);
 	OffsetCurveSetBuilder curveSetBuilder(*g, distance, curveBuilder);
 
 	std::vector<SegmentString*>& bufferSegStrList=curveSetBuilder.getCurves();
@@ -161,7 +386,7 @@ for (size_t i = 0, n=bufferSegStrList.size(); i<n; i++)
 #endif
 
 	Geometry* resultGeom=NULL;
-	std::vector<Geometry*> *resultPolyList=NULL;
+	std::auto_ptr< std::vector<Geometry*> > resultPolyList;
 	std::vector<BufferSubgraph*> subgraphList;
 
 	try {
@@ -178,7 +403,7 @@ for (size_t i = 0, n=bufferSegStrList.size(); i<n; i++)
 #endif
 		PolygonBuilder polyBuilder(geomFact);
 		buildSubgraphs(subgraphList, polyBuilder);
-		resultPolyList=polyBuilder.getPolygons();
+		resultPolyList.reset( polyBuilder.getPolygons() );
 #if GEOS_DEBUG
 	std::cerr << "PolygonBuilder got " << resultPolyList->size()
 	          << " polygons" << std::endl;
@@ -195,7 +420,8 @@ for (size_t i = 0, n=bufferSegStrList.size(); i<n; i++)
 			return createEmptyResultGeometry();
 		}
 
-		resultGeom=geomFact->buildGeometry(resultPolyList);
+		// resultPolyList ownership transferred here
+		resultGeom=geomFact->buildGeometry(resultPolyList.release());
 	} catch (const util::GEOSException& /* exc */) {
 		for (size_t i=0, n=subgraphList.size(); i<n; i++)
 			delete subgraphList[i];
@@ -255,7 +481,7 @@ void
 BufferBuilder::computeNodedEdges(SegmentString::NonConstVect& bufferSegStrList,
 		const PrecisionModel *precisionModel) // throw(GEOSException)
 {
-	Noder *noder = getNoder(precisionModel);
+	Noder* noder = getNoder( precisionModel );
 
 	noder->computeNodes(&bufferSegStrList);
 
@@ -284,7 +510,7 @@ BufferBuilder::computeNodedEdges(SegmentString::NonConstVect& bufferSegStrList,
 		Edge* edge = new Edge(cs, new Label(*oldLabel));
 
 		// will take care of the Edge ownership
-		insertEdge(edge);
+		insertUniqueEdge(edge);
 	}
 
 	if ( nodedSegStrings != &bufferSegStrList )
@@ -297,7 +523,7 @@ BufferBuilder::computeNodedEdges(SegmentString::NonConstVect& bufferSegStrList,
 
 /*private*/
 void
-BufferBuilder::insertEdge(Edge *e)
+BufferBuilder::insertUniqueEdge(Edge *e)
 {
 	//<FIX> MD 8 Oct 03  speed up identical edge lookup
 	// fast lookup
