@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: OffsetCurveSetBuilder.cpp 2131 2008-07-15 22:04:51Z mloskot $
+ * $Id: OffsetCurveSetBuilder.cpp 2655 2009-10-05 18:40:11Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -14,7 +14,7 @@
  *
  **********************************************************************
  *
- * Last port: operation/buffer/OffsetCurveSetBuilder.java rev. 1.7 (JTS-1.7)
+ * Last port: operation/buffer/OffsetCurveSetBuilder.java rev. 1.11 (JTS-1.10)
  *
  **********************************************************************/
 
@@ -35,7 +35,7 @@
 #include <geos/geom/Triangle.h>
 #include <geos/geomgraph/Position.h>
 #include <geos/geomgraph/Label.h>
-#include <geos/noding/SegmentString.h>
+#include <geos/noding/NodedSegmentString.h>
 
 #include <cmath>
 #include <cassert>
@@ -68,11 +68,11 @@ OffsetCurveSetBuilder::OffsetCurveSetBuilder(const Geometry& newInputGeom,
 
 OffsetCurveSetBuilder::~OffsetCurveSetBuilder()
 {
-	//delete cga;
 	for (size_t i=0, n=curveList.size(); i<n; ++i)
 	{
-		delete curveList[i]->getCoordinates();
-		delete curveList[i];
+		SegmentString* ss = curveList[i];
+		delete ss->getCoordinates();
+		delete ss;
 	}
 	for (size_t i=0, n=newLabels.size(); i<n; ++i)
 		delete newLabels[i];
@@ -93,7 +93,7 @@ OffsetCurveSetBuilder::addCurves(const std::vector<CoordinateSequence*>& lineLis
 {
 	for (size_t i=0, n=lineList.size(); i<n; ++i)
 	{
-		CoordinateSequence *coords=lineList[i];
+		CoordinateSequence *coords = lineList[i];
 		addCurve(coords, leftLoc, rightLoc);
 	}
 }
@@ -117,7 +117,11 @@ OffsetCurveSetBuilder::addCurve(CoordinateSequence *coord,
 
 	// add the edge for a coordinate list which is a raw offset curve
 	Label *newlabel = new Label(0, Location::BOUNDARY, leftLoc, rightLoc);
-	SegmentString *e=new SegmentString(coord, newlabel); // SegmentString doesnt own the sequence, so we need to delete in the destructor
+
+	SegmentString *e=new NodedSegmentString(coord, newlabel);
+
+	// SegmentString doesnt own the sequence, so we need to delete in
+	// the destructor
 	newLabels.push_back(newlabel);
 	curveList.push_back(e);
 }
@@ -222,6 +226,14 @@ OffsetCurveSetBuilder::addPolygon(const Polygon *p)
 		return;
 	}
 
+	// don't attemtp to buffer a polygon
+	// with too few distinct vertices
+	if (distance <= 0.0 && shellCoord->size() < 3)
+	{
+		delete shellCoord;
+        	return;
+	}
+
 	addPolygonRing(
 		shellCoord,
 		offsetDistance,
