@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: LineMerger.cpp 2231 2009-01-05 23:42:08Z swongu $
+ * $Id: LineMerger.cpp 2733 2009-11-20 19:58:33Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -12,6 +12,10 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: operation/linemerge/LineMerger.java rev. 1.7 (JTS-1.10)
+ *
  **********************************************************************/
 
 #include <geos/operation/linemerge/LineMerger.h>
@@ -20,6 +24,7 @@
 #include <geos/planargraph/DirectedEdge.h>
 #include <geos/planargraph/Edge.h>
 #include <geos/planargraph/Node.h>
+//#include <geos/planargraph/GraphComponent.h>
 #include <geos/geom/GeometryComponentFilter.h>
 #include <geos/geom/LineString.h>
 
@@ -98,6 +103,16 @@ LineMerger::merge()
 {
 	if (mergedLineStrings!=NULL) return;
 
+	// reset marks (this allows incremental processing)
+	GraphComponent::setMarkedMap(graph.nodeIterator(), graph.nodeEnd(),
+	                                                              false);
+	GraphComponent::setMarked(graph.edgeIterator(), graph.edgeEnd(),
+	                                                              false);
+
+	for (size_t i=0, n=edgeStrings.size(); i<n; ++i) 
+		delete edgeStrings[i];
+	edgeStrings.clear();
+
 	buildEdgeStringsForObviousStartNodes();
 	buildEdgeStringsForIsolatedLoops();
 
@@ -128,9 +143,12 @@ LineMerger::buildEdgeStringsForUnprocessedNodes()
 #if GEOS_DEBUG
 	cerr<<__FUNCTION__<<endl;
 #endif
-	vector<Node*> *nodes=graph.getNodes();
-	for (size_t i=0; i<nodes->size(); ++i) {
-		Node *node=(*nodes)[i];
+	typedef std::vector<Node*> Nodes;
+
+	Nodes nodes;
+	graph.getNodes(nodes);
+	for (Nodes::size_type i=0, in=nodes.size(); i<in; ++i) {
+		Node *node=nodes[i];
 #if GEOS_DEBUG
 		cerr<<"Node "<<i<<": "<<*node<<endl;
 #endif
@@ -143,7 +161,6 @@ LineMerger::buildEdgeStringsForUnprocessedNodes()
 #endif
 		}
 	}
-	delete nodes;
 }
 
 void
@@ -152,10 +169,12 @@ LineMerger::buildEdgeStringsForNonDegree2Nodes()
 #if GEOS_DEBUG
 	cerr<<__FUNCTION__<<endl;
 #endif
-	vector<Node*> *nodes=graph.getNodes();
-	size_t size=nodes->size();
-	for (size_t i=0; i<size; i++) {
-		Node *node=(*nodes)[i];
+	typedef std::vector<Node*> Nodes;
+
+	Nodes nodes;
+	graph.getNodes(nodes);
+	for (Nodes::size_type i=0, in=nodes.size(); i<in; ++i) {
+		Node *node=nodes[i];
 #if GEOS_DEBUG
 		cerr<<"Node "<<i<<": "<<*node<<endl;
 #endif
@@ -167,7 +186,6 @@ LineMerger::buildEdgeStringsForNonDegree2Nodes()
 #endif
 		}
 	}
-	delete nodes;
 }
 
 void
