@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: OverlayOp.cpp 2220 2008-11-11 17:32:38Z pramsey $
+ * $Id: OverlayOp.cpp 2763 2009-12-03 19:02:17Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -14,12 +14,14 @@
  *
  ***********************************************************************
  *
- * Last port: operation/overlay/OverlayOp.java rev. 1.23
+ * Last port: operation/overlay/OverlayOp.java rev. 1.31 (JTS-1.10)
+ *
+ * NOTE: Use of EdgeNodingValidator is not strictly the same
  *
  **********************************************************************/
 
 #include <geos/operation/overlay/OverlayOp.h>
-#include <geos/operation/overlay/OverlayResultValidator.h>
+#include <geos/operation/overlay/validate/OverlayResultValidator.h>
 #include <geos/operation/overlay/ElevationMatrix.h>
 #include <geos/operation/overlay/OverlayNodeFactory.h>
 #include <geos/operation/overlay/PolygonBuilder.h>
@@ -680,14 +682,28 @@ OverlayOp::computeOverlay(OverlayOp::OpCode opCode)
 	//Debug.println(edgeList);
 
 #ifdef ENABLE_EDGE_NODING_VALIDATOR // {
-	if ( resultPrecisionModel->isFloating() )
+	/**
+	 * Check that the noding completed correctly.
+	 *
+	 * This test is slow, but necessary in order to catch
+	 * robustness failure situations.
+	 * If an exception is thrown because of a noding failure,
+	 * then snapping will be performed, which will hopefully avoid
+	 * the problem.
+	 * In the future hopefully a faster check can be developed.
+	 *
+	 */
+	if ( resultPrecisionModel->isFloating() ) // NOTE: this is not in JTS
 	{
-		// Will throw TopologyException if noding is found to be invalid
-		EdgeNodingValidator nv(edgeList.getEdges());
 
-		try {
-			nv.checkValid();
-		} catch (const util::TopologyException& ex) {
+		try
+		{
+			// Will throw TopologyException if noding is
+			// found to be invalid
+			EdgeNodingValidator::checkValid(edgeList.getEdges());
+		}
+		catch (const util::TopologyException& ex)
+		{
 #ifdef GEOS_DEBUG_VALIDATION // {
 			cout << "EdgeNodingValidator found noding invalid: " << ex.what() << endl;
 #endif // }
@@ -895,6 +911,11 @@ struct PointCoveredByAny: public geom::CoordinateFilter
 			"A point on first geom boundary isn't covered "
 			"by either result or second geom");
 	}
+
+private:
+    // Declare type as noncopyable
+    PointCoveredByAny(const PointCoveredByAny& other);
+    PointCoveredByAny& operator=(const PointCoveredByAny& rhs);
 };
 
 void
@@ -955,14 +976,14 @@ OverlayOp::checkObviouslyWrongResult(OverlayOp::OpCode opCode)
 	// Add your tests here
 
 #else
-    UNREFERENCED_PARAMETER(opCode);
+    ::geos::ignore_unused_variable_warning(opCode);
 #endif
 
 #ifdef ENABLE_OVERLAY_RESULT_VALIDATOR
 	// This only work for FLOATING precision
 	if ( resultPrecisionModel->isFloating() )
 	{
-		OverlayResultValidator validator( *(arg[0]->getGeometry()),
+		validate::OverlayResultValidator validator( *(arg[0]->getGeometry()),
 			*(arg[1]->getGeometry()), *(resultGeom));
 		bool isvalid = validator.isValid(opCode);
 		if ( ! isvalid )
