@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: PolygonizeGraph.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: PolygonizeGraph.cpp 2733 2009-11-20 19:58:33Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -12,6 +12,10 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: operation/polygonize/PolygonizeGraph.java rev. 1.6 (JTS-1.10)
+ *
  **********************************************************************/
 
 #include <geos/operation/polygonize/PolygonizeGraph.h>
@@ -31,6 +35,9 @@
 using namespace geos::planargraph;
 using namespace geos::geom;
 
+// Define the following to add assertions on downcasts
+//#define GEOS_CAST_PARANOIA 1
+
 namespace geos {
 namespace operation { // geos.operation
 namespace polygonize { // geos.operation.polygonize
@@ -155,78 +162,63 @@ PolygonizeGraph::getNode(const Coordinate& pt)
 void
 PolygonizeGraph::computeNextCWEdges()
 {
-	std::vector<Node*> *pns=getNodes();
+	typedef std::vector<Node*> Nodes;
+	Nodes pns; getNodes(pns);
 	// set the next pointers for the edges around each node
-	for(int i=0;i<(int)pns->size();i++) {
-		Node *node=(*pns)[i];
+	for(Nodes::size_type i=0, in=pns.size(); i<in; ++i) {
+		Node *node=pns[i];
 		computeNextCWEdges(node);
 	}
-	delete pns;
 }
 
-/*
- * Convert the maximal edge rings found by the initial graph traversal
- * into the minimal edge rings required by JTS polygon topology rules.
- *
- * @param ringEdges the list of start edges for the edgeRings to convert.
- */
+/* private */
 void
-PolygonizeGraph::convertMaximalToMinimalEdgeRings(std::vector<PolygonizeDirectedEdge*> *ringEdges)
+PolygonizeGraph::convertMaximalToMinimalEdgeRings(
+		std::vector<PolygonizeDirectedEdge*> &ringEdges)
 {
-	for(int i=0;i<(int)ringEdges->size();i++)
-	{
-		PolygonizeDirectedEdge *de=(*ringEdges)[i];
-		long label=de->getLabel();
-		std::vector<Node*> *intNodes=findIntersectionNodes(de, label);
-		if (intNodes==NULL) continue;
+	typedef std::vector<Node*> IntersectionNodes;
+	typedef std::vector<PolygonizeDirectedEdge*> RingEdges;
 
-		// flip the next pointers on the intersection nodes to
-		// create minimal edge rings
-		//std::vector<Node*> *pns=getNodes();
+	IntersectionNodes intNodes;
+	for(RingEdges::size_type i=0, in=ringEdges.size();
+			i<in; ++i)
+	{
+		PolygonizeDirectedEdge *de = ringEdges[i];
+		long label=de->getLabel();
+		findIntersectionNodes(de, label, intNodes);
 
 		// set the next pointers for the edges around each node
-		for(int j=0;j<(int)intNodes->size();j++) {
-			Node *node=(*intNodes)[j];
+		for(IntersectionNodes::size_type j=0, jn=intNodes.size();
+				j<jn; ++j)
+		{
+			Node *node=intNodes[j];
 			computeNextCCWEdges(node, label);
 		}
 
-		delete intNodes;
+		intNodes.clear(); 
 	}
 }
 
-/*
- * Finds all nodes in a maximal edgering which are self-intersection nodes
- * @param startDE
- * @param label
- * @return the list of intersection nodes found,
- * or <code>NULL</code> if no intersection nodes were found.
- * Ownership of returned object goes to caller.
- */
-std::vector<Node*>*
-PolygonizeGraph::findIntersectionNodes(PolygonizeDirectedEdge *startDE, long label)
+/* private static */
+void
+PolygonizeGraph::findIntersectionNodes(PolygonizeDirectedEdge *startDE,
+		long label, std::vector<Node*>& intNodes)
 {
 	PolygonizeDirectedEdge *de=startDE;
-	std::vector<Node*> *intNodes=NULL;
 	do {
 		Node *node=de->getFromNode();
 		if (getDegree(node, label) > 1) {
-			if (intNodes==NULL)
-				intNodes=new std::vector<Node*>();
-			intNodes->push_back(node);
+			intNodes.push_back(node);
 		}
 		de=de->getNext();
 		assert(de!=NULL); // found NULL DE in ring
 		assert(de==startDE || !de->isInRing()); // found DE already in ring
 	} while (de!=startDE);
-	return intNodes;
 }
 
-/**
- * Computes the EdgeRings formed by the edges in this graph.
- * @return a list of the EdgeRing found by the polygonization process.
- */
-std::vector<EdgeRing*>*
-PolygonizeGraph::getEdgeRings()
+/* public */
+void
+PolygonizeGraph::getEdgeRings(std::vector<EdgeRing*>& edgeRingList)
 {
 	// maybe could optimize this, since most of these pointers should
 	// be set correctly already
@@ -235,78 +227,103 @@ PolygonizeGraph::getEdgeRings()
 
 	// clear labels of all edges in graph
 	label(dirEdges, -1);
-	std::vector<PolygonizeDirectedEdge*> *maximalRings=findLabeledEdgeRings(dirEdges);
+	std::vector<PolygonizeDirectedEdge*> maximalRings;
+	findLabeledEdgeRings(dirEdges, maximalRings);
 	convertMaximalToMinimalEdgeRings(maximalRings);
-	delete maximalRings;
+	maximalRings.clear(); // not needed anymore
 
 	// find all edgerings
-	std::vector<EdgeRing*> *edgeRingList=new std::vector<EdgeRing*>();
 	for(unsigned int i=0; i<dirEdges.size(); ++i)
 	{
 		PolygonizeDirectedEdge *de=(PolygonizeDirectedEdge*)dirEdges[i];
 		if (de->isMarked()) continue;
 		if (de->isInRing()) continue;
 		EdgeRing *er=findEdgeRing(de);
-		edgeRingList->push_back(er);
+		edgeRingList.push_back(er);
 	}
-	return edgeRingList;
 }
 
-/**
-*
-* @param dirEdges a List of the DirectedEdges in the graph
-* @return a List of DirectedEdges, one for each edge ring found
-*/
-std::vector<PolygonizeDirectedEdge*>*
-PolygonizeGraph::findLabeledEdgeRings(std::vector<DirectedEdge*> &dirEdges)
+/* static private */
+void
+PolygonizeGraph::findLabeledEdgeRings(std::vector<DirectedEdge*> &dirEdges,
+		std::vector<PolygonizeDirectedEdge*> &edgeRingStarts)
 {
-	std::vector<PolygonizeDirectedEdge*> *edgeRingStarts=new std::vector<PolygonizeDirectedEdge*>();
+	typedef std::vector<DirectedEdge*> Edges;
+
+	Edges edges;
+
 	// label the edge rings formed
 	long currLabel=1;
-	for(unsigned int i=0; i<dirEdges.size(); ++i)
+	for(Edges::size_type i=0, n=dirEdges.size(); i<n; ++i)
 	{
-		PolygonizeDirectedEdge *de=(PolygonizeDirectedEdge*)dirEdges[i];
+#ifdef GEOS_CAST_PARANOIA
+		assert(dynamic_cast<PolygonizeDirectedEdge*>(dirEdges[i]));
+#endif
+		PolygonizeDirectedEdge *de =
+			static_cast<PolygonizeDirectedEdge*>(dirEdges[i]);
+
 		if (de->isMarked()) continue;
 		if (de->getLabel() >= 0) continue;
-		edgeRingStarts->push_back(de);
-		std::vector<DirectedEdge*> *edges=findDirEdgesInRing(de);
-		label(*edges, currLabel);
-		delete edges;
+		edgeRingStarts.push_back(de);
+
+		findDirEdgesInRing(de, edges);
+		label(edges, currLabel);
+		edges.clear(); 
+
 		++currLabel;
 	}
-	return edgeRingStarts;
 }
 
-/*
- * Finds and removes all cut edges from the graph.
- * @return a list of the LineString forming the removed cut edges
- */
-std::vector<const LineString*> *
-PolygonizeGraph::deleteCutEdges()
+/* public */
+void
+PolygonizeGraph::deleteCutEdges(std::vector<const LineString*> &cutLines)
 {
 	computeNextCWEdges();
 
+	typedef std::vector<PolygonizeDirectedEdge*> DirEdges;
+
 	// label the current set of edgerings
-	delete findLabeledEdgeRings(dirEdges);
+	DirEdges junk;
+	findLabeledEdgeRings(dirEdges, junk);
+	junk.clear(); // not needed anymore
 
 	/*
 	 * Cut Edges are edges where both dirEdges have the same label.
 	 * Delete them, and record them
 	 */
-	std::vector<const LineString*> *cutLines=new std::vector<const LineString*>();
-	for(unsigned int i=0; i<dirEdges.size(); ++i) {
-		PolygonizeDirectedEdge *de=(PolygonizeDirectedEdge*)dirEdges[i];
+	for (DirEdges::size_type i=0, in=dirEdges.size(); i<in; ++i)
+	{
+		DirectedEdge *de_ = dirEdges[i];
+#ifdef GEOS_CAST_PARANOIA
+		assert(dynamic_cast<PolygonizeDirectedEdge*>(de_));
+#endif
+		PolygonizeDirectedEdge *de =
+			static_cast<PolygonizeDirectedEdge*>(de_);
+
 		if (de->isMarked()) continue;
-		PolygonizeDirectedEdge *sym=(PolygonizeDirectedEdge*) de->getSym();
-		if (de->getLabel()==sym->getLabel()) {
+
+		DirectedEdge *sym_ = de->getSym();
+#ifdef GEOS_CAST_PARANOIA
+		assert(dynamic_cast<PolygonizeDirectedEdge*>(sym_));
+#endif
+		PolygonizeDirectedEdge *sym =
+			static_cast<PolygonizeDirectedEdge*>(sym_);
+
+		if (de->getLabel()==sym->getLabel())
+		{
 			de->setMarked(true);
 			sym->setMarked(true);
+
 			// save the line as a cut edge
-			PolygonizeEdge *e=(PolygonizeEdge*) de->getEdge();
-			cutLines->push_back(e->getLine());
+			Edge *e_ = de->getEdge();
+#ifdef GEOS_CAST_PARANOIA
+			assert(dynamic_cast<PolygonizeEdge*>(e_));
+#endif
+			PolygonizeEdge *e = static_cast<PolygonizeEdge*>(e_);
+
+			cutLines.push_back(e->getLine());
 		}
 	}
-	return cutLines;
 }
 
 void
@@ -392,26 +409,18 @@ PolygonizeGraph::computeNextCCWEdges(Node *node, long label)
 	}
 }
 
-/*
- * Traverse a ring of DirectedEdges, accumulating them into a list.
- * This assumes that all dangling directed edges have been removed
- * from the graph, so that there is always a next dirEdge.
- *
- * @param startDE the DirectedEdge to start traversing at
- * @return a List of DirectedEdges that form a ring
- */
-std::vector<DirectedEdge*>*
-PolygonizeGraph::findDirEdgesInRing(PolygonizeDirectedEdge *startDE)
+/* static private */
+void
+PolygonizeGraph::findDirEdgesInRing(PolygonizeDirectedEdge *startDE,
+		std::vector<DirectedEdge*>& edges)
 {
 	PolygonizeDirectedEdge *de=startDE;
-	std::vector<DirectedEdge*> *edges=new std::vector<DirectedEdge*>();
 	do {
-		edges->push_back(de);
+		edges.push_back(de);
 		de=de->getNext();
 		assert(de != NULL); // found NULL DE in ring
 		assert(de==startDE || !de->isInRing()); // found DE already in ring
 	} while (de != startDE);
-	return edges;
 }
 
 EdgeRing *
@@ -431,21 +440,11 @@ PolygonizeGraph::findEdgeRing(PolygonizeDirectedEdge *startDE)
 	return er;
 }
 
-/**
- * Marks all edges from the graph which are "dangles".
- * Dangles are which are incident on a node with degree 1.
- * This process is recursive, since removing a dangling edge
- * may result in another edge becoming a dangle.
- * In order to handle large recursion depths efficiently,
- * an explicit recursion stack is used
- *
- * @return a List containing the LineStrings that formed dangles
- */
-std::vector<const LineString*>*
-PolygonizeGraph::deleteDangles()
+/* public */
+void
+PolygonizeGraph::deleteDangles(std::vector<const LineString*>& dangleLines)
 {
 	std::vector<Node*> *nodesToRemove=findNodesOfDegree(1);
-	std::vector<const LineString*> *dangleLines=new std::vector<const LineString*>();
 	std::vector<Node*> nodeStack;
 	for(int i=0;i<(int)nodesToRemove->size();i++) {
 		nodeStack.push_back((*nodesToRemove)[i]);
@@ -466,14 +465,13 @@ PolygonizeGraph::deleteDangles()
 				sym->setMarked(true);
 			// save the line as a dangle
 			PolygonizeEdge *e=(PolygonizeEdge*) de->getEdge();
-			dangleLines->push_back(e->getLine());
+			dangleLines.push_back(e->getLine());
 			Node *toNode=de->getToNode();
 			// add the toNode to the list to be processed, if it is now a dangle
 			if (getDegreeNonDeleted(toNode)==1)
 				nodeStack.push_back(toNode);
 		}
 	}
-	return dangleLines;
 }
 
 } // namespace geos.operation.polygonize
