@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: Polygonizer.cpp 1997 2007-08-22 23:45:12Z mloskot $
+ * $Id: Polygonizer.cpp 2730 2009-11-19 20:29:01Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -12,6 +12,10 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: operation/polygonize/Polygonizer.java rev. 1.6 (JTS-1.10)
+ *
  **********************************************************************/
 
 #include <geos/operation/polygonize/Polygonizer.h>
@@ -227,16 +231,21 @@ Polygonizer::polygonize()
 
 	polyList=new vector<Polygon*>();
 
-	if (graph==NULL) // No valid geometries added
-	{
-		return; 
-	}
+	// if no geometries were supplied it's possible graph could be null
+	if (graph==NULL) return; 
 
-	dangles=graph->deleteDangles();
-	cutEdges=graph->deleteCutEdges();
-	vector<EdgeRing*> *edgeRingList=graph->getEdgeRings();
+	// TODO: drop this heap allocation
+	dangles = new std::vector<const LineString*>();
+	graph->deleteDangles(*dangles);
+
+	// TODO: drop this heap allocation
+	cutEdges = new std::vector<const LineString*>();
+	graph->deleteCutEdges(*cutEdges);
+
+	vector<EdgeRing*> edgeRingList;
+	graph->getEdgeRings(edgeRingList);
 #if GEOS_DEBUG
-	cerr<<"Polygonizer::polygonize(): "<<edgeRingList->size()<<" edgeRings in graph"<<endl;
+	cerr<<"Polygonizer::polygonize(): "<<edgeRingList.size()<<" edgeRings in graph"<<endl;
 #endif
 	vector<EdgeRing*> *validEdgeRingList=new vector<EdgeRing*>();
 	invalidRingLines=new vector<LineString*>();
@@ -245,7 +254,6 @@ Polygonizer::polygonize()
 	cerr<<"                           "<<validEdgeRingList->size()<<" valid"<<endl;
 	cerr<<"                           "<<invalidRingLines->size()<<" invalid"<<endl;
 #endif
-	delete edgeRingList;
 
 	findShellsAndHoles(validEdgeRingList);
 #if GEOS_DEBUG
@@ -263,12 +271,17 @@ Polygonizer::polygonize()
 	delete validEdgeRingList;
 }
 
+/* private */
 void
-Polygonizer::findValidRings(vector<EdgeRing*> *edgeRingList, vector<EdgeRing*> *validEdgeRingList, vector<LineString*> *invalidRingList)
+Polygonizer::findValidRings(vector<EdgeRing*>& edgeRingList,
+	vector<EdgeRing*> *validEdgeRingList,
+	vector<LineString*> *invalidRingList)
 {
-	for (unsigned int i=0, n=edgeRingList->size(); i<n; ++i)
+	typedef vector<EdgeRing*> EdgeRingList;
+	
+	for (EdgeRingList::size_type i=0, n=edgeRingList.size(); i<n; ++i)
 	{
-		EdgeRing *er=(*edgeRingList)[i];
+		EdgeRing *er = edgeRingList[i];
 		if (er->isValid())
 			validEdgeRingList->push_back(er);
 		else
