@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: RectangleIntersects.cpp 2066 2007-12-21 20:42:34Z benjubb $
+ * $Id: RectangleIntersects.cpp 2763 2009-12-03 19:02:17Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -13,7 +13,7 @@
  *
  **********************************************************************
  *
- * Last port: operation/predicate/RectangleIntersects.java rev 1.3 (JTS-1.7)
+ * Last port: operation/predicate/RectangleIntersects.java rev 1.4 (JTS-1.10)
  *
  **********************************************************************/
 
@@ -51,6 +51,10 @@ private:
 	const geom::Envelope &rectEnv;
 	bool intersectsVar;
 
+    // Declare type as noncopyable
+    EnvelopeIntersectsVisitor(const EnvelopeIntersectsVisitor& other);
+    EnvelopeIntersectsVisitor& operator=(const EnvelopeIntersectsVisitor& rhs);
+
 protected:
 
 	/**
@@ -130,6 +134,10 @@ private:
 	bool containsPointVar;
 	const geom::CoordinateSequence &rectSeq;
 
+    // Declare type as noncopyable
+    ContainsPointVisitor(const ContainsPointVisitor& other);
+    ContainsPointVisitor& operator=(const ContainsPointVisitor& rhs);
+
 protected:
 
 	void visit(const geom::Geometry &geom)
@@ -138,7 +146,7 @@ protected:
 
 		const geom::Polygon *poly;
 
-		if ( !(poly=dynamic_cast<const geom::Polygon *>(&geom)) ) {
+		if ( 0 == (poly=dynamic_cast<const geom::Polygon *>(&geom)) ) {
 			return;
 		}
 
@@ -212,6 +220,10 @@ private:
 		}
 	}
 
+    // Declare type as noncopyable
+    LineIntersectsVisitor(const LineIntersectsVisitor& other);
+    LineIntersectsVisitor& operator=(const LineIntersectsVisitor& rhs);
+
 protected:
 
 	void visit(const geom::Geometry &geom)
