@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: IsValidOp.cpp 2210 2008-10-23 17:33:16Z warmerdam $
+ * $Id: IsValidOp.cpp 2763 2009-12-03 19:02:17Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -14,13 +14,15 @@
  *
  **********************************************************************
  *
- * Last port: operation/valid/IsValidOp.java rev. 1.39 (JTS-1.7)
+ * Last port: operation/valid/IsValidOp.java rev. 1.41 (JTS-1.10)
  *
  **********************************************************************/
 
+#include "IndexedNestedRingTester.h"
+
 #include <geos/operation/valid/IsValidOp.h>
 #include <geos/operation/valid/ConsistentAreaTester.h>
-#include <geos/operation/valid/QuadtreeNestedRingTester.h>
+#include <geos/operation/valid/ConnectedInteriorTester.h>
 #include <geos/operation/valid/ConnectedInteriorTester.h>
 #include <geos/util/UnsupportedOperationException.h>
 #include <geos/geomgraph/index/SegmentIntersector.h> 
@@ -125,7 +127,7 @@ IsValidOp::checkValid(const Geometry *g)
 	else if (typeid(*g)==typeid(LineString)) checkValid((LineString*)g);
 	else if (typeid(*g)==typeid(Polygon)) checkValid((Polygon*)g);
 	else if (typeid(*g)==typeid(MultiPolygon)) checkValid((MultiPolygon*)g);
-	else if ((gc=dynamic_cast<const GeometryCollection *>(g)))
+	else if (0 != (gc=dynamic_cast<const GeometryCollection *>(g)))
 		checkValid(gc);
 	else throw util::UnsupportedOperationException();
         isChecked=true;
@@ -401,7 +403,8 @@ IsValidOp::checkHolesNotNested(const Polygon *p, GeometryGraph *graph)
 {
 	//SimpleNestedRingTester nestedTester(graph);
 	//SweeplineNestedRingTester nestedTester(graph);
-	QuadtreeNestedRingTester nestedTester(graph);
+	//QuadtreeNestedRingTester nestedTester(graph);
+	IndexedNestedRingTester nestedTester(graph);
 
 	int nholes=p->getNumInteriorRing();
 	for(int i=0; i<nholes; ++i)
