@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: EnhancedPrecisionOp.cpp 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: EnhancedPrecisionOp.cpp 2755 2009-11-30 17:29:48Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -82,6 +82,8 @@ EnhancedPrecisionOp::intersection(
 #if GEOS_DEBUG
 		std::cerr << "Reduced operation exception: "
 		          << ex2.what() << std::endl;
+#else
+        (void)ex2;
 #endif
 		throw originalEx;
 	}
