@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: DouglasPeuckerSimplifier.cpp 2127 2008-05-20 21:25:21Z mloskot $
+ * $Id: DouglasPeuckerSimplifier.cpp 2757 2009-12-01 15:39:41Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -103,7 +103,7 @@ DPTransformer::transformCoordinates(
 		const CoordinateSequence* coords,
 		const Geometry* parent)
 {
-    UNREFERENCED_PARAMETER(parent);
+    ::geos::ignore_unused_variable_warning(parent);
 
 	const Coordinate::Vect* inputPts = coords->toVector();
 	assert(inputPts);
