@@ -1,4 +1,4 @@
-# $Id: runtests.py,v 1.2 2004/07/26 15:40:58 sgillies Exp $
+# $Id: runtests.py 1820 2006-09-06 16:54:23Z mloskot $
 # ===========================================================================
 # Copyright 2004 Sean Gillies, sgillies@frii.com
 #
@@ -17,16 +17,13 @@
 import unittest
 
 # Import test cases
-from cases.pointtest import PointTestCase
-from cases.wkttest import WKTReaderTestCase, WKTWriterTestCase
+from test_geometry import TestGeometry
 
 # Create a test suite
 suite = unittest.TestSuite()
 
 # Add tests to the suite
-suite.addTest(PointTestCase)
-suite.addTest(WKTReaderTestCase)
-suite.addTest(WKTWriterTestCase)
+suite.addTest(TestGeometry)
 
 # If module is run as a script, execute every test case in the suite
 if __name__ == '__main__':
