@@ -1,10 +1,10 @@
-// $Id: isCCWTest.cpp 2165 2008-08-18 23:45:34Z mloskot $
+// $Id: isCCWTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for CGAlgorithms::isCCW() function
 // Ported from JTS junit/algorithm/IsCCWTest.java
 
 // tut
-#include <tut.h>
+#include <tut.hpp>
 // geos
 #include <geos/algorithm/CGAlgorithms.h>
 #include <geos/geom/Polygon.h>
