@@ -1,7 +1,10 @@
-// $Id: GeometryFactoryTest.cpp 2162 2008-08-18 16:30:56Z mloskot $
+// $Id: GeometryFactoryTest.cpp 2761 2009-12-03 18:56:25Z mloskot $
 // 
 // Test Suite for geos::geom::GeometryFactory class.
 
+// tut
+#include <tut.hpp>
+#include <utility.h>
 // geos
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/GeometryCollection.h>
@@ -23,9 +26,6 @@
 // std
 #include <vector>
 #include <cstring> // std::size_t
-// tut
-#include <tut.h>
-#include <utility.h>
 
 /*!
  * \brief
@@ -46,22 +46,26 @@ namespace tut
 	//
 
 	// Common data used by tests
-	struct test_geometryfactory_data
-	{
-		// Test data
-		const int x_;
-		const int y_;
-		const int z_;
+    struct test_geometryfactory_data
+    {
+        // Test data
+        const int x_;
+        const int y_;
+        const int z_;
 
-		const int srid_;
-		geos::geom::PrecisionModel pm_;
-		geos::geom::GeometryFactory factory_;
-		geos::io::WKTReader reader_;
+        const int srid_;
+        geos::geom::PrecisionModel pm_;
+        geos::geom::GeometryFactory factory_;
+        geos::io::WKTReader reader_;
 
-		test_geometryfactory_data()
-			: x_(5), y_(10), z_(15), srid_(666), pm_(1.0), factory_(&pm_, srid_), reader_(&factory_)
-		{}
-	};
+        test_geometryfactory_data()
+            : x_(5), y_(10), z_(15), srid_(666), pm_(1.0), factory_(&pm_, srid_), reader_(&factory_)
+        {}
+    private:
+        // Declare type as noncopyable
+        test_geometryfactory_data(const test_geometryfactory_data& other);
+        test_geometryfactory_data& operator=(const test_geometryfactory_data& rhs);
+    };
 
 	typedef test_group<test_geometryfactory_data> group;
 	typedef group::object object;
@@ -701,7 +705,7 @@ namespace tut
 		ensure_equals( exterior->getBoundaryDimension(), geos::geom::Dimension::False );
 		ensure_equals( exterior->getNumPoints(), size );
 		ensure_equals( exterior->getArea(), 0.0 );
-		ensure_not_equals( exterior->getLength(), 0.0 );
+		ensure( exterior->getLength() != 0.0 );
 
 		// Create polygon
 		PolygonPtr poly = factory_.createPolygon(exterior, 0);
@@ -712,8 +716,8 @@ namespace tut
 		ensure_equals( poly->getDimension(), geos::geom::Dimension::A );
 		ensure_equals( poly->getBoundaryDimension(), geos::geom::Dimension::L );
 		ensure_equals( poly->getNumPoints(), size );
-		ensure_not_equals( poly->getArea(), 0.0 );
-		ensure_not_equals( poly->getLength(), 0.0 );
+		ensure( poly->getArea() != 0.0 );
+		ensure( poly->getLength() != 0.0 );
 
 		// FREE MEMORY
 		factory_.destroyGeometry(poly);	
@@ -750,7 +754,7 @@ namespace tut
 		ensure_equals( exterior->getBoundaryDimension(), geos::geom::Dimension::False );
 		ensure_equals( exterior->getNumPoints(), exteriorSize );
 		ensure_equals( exterior->getArea(), 0.0 );
-		ensure_not_equals( exterior->getLength(), 0.0 );
+		ensure( exterior->getLength() != 0.0 );
 
 		// Create collection of holes
 		GeometryPtr geo = reader_.read(("LINEARRING(7 7, 12 7, 12 12, 7 12, 7 7)"));
@@ -773,8 +777,8 @@ namespace tut
 		ensure_equals( poly->getDimension(), geos::geom::Dimension::A );
 		ensure_equals( poly->getBoundaryDimension(), geos::geom::Dimension::L );
 		ensure_equals( poly->getNumPoints(), exteriorSize + interiorSize );
-		ensure_not_equals( poly->getArea(), 0.0 );
-		ensure_not_equals( poly->getLength(), 0.0 );
+		ensure( poly->getArea() != 0.0 );
+		ensure( poly->getLength() != 0.0 );
 
 		ensure_equals( poly->getNumGeometries(), 1u );
 		ensure_equals( poly->getNumInteriorRing(), 1u );
