@@ -1,7 +1,10 @@
-// $Id: MultiPointTest.cpp 2162 2008-08-18 16:30:56Z mloskot $
+// $Id: MultiPointTest.cpp 2761 2009-12-03 18:56:25Z mloskot $
 // 
 // Test Suite for geos::geom::MultiPoint class.
 
+// tut
+#include <tut.hpp>
+#include <utility.h>
 // geos
 #include <geos/geom/MultiPoint.h>
 #include <geos/geom/Dimension.h>
@@ -12,9 +15,6 @@
 #include <geos/io/WKTReader.h>
 // std
 #include <cmath>
-// tut
-#include <tut.h>
-#include <utility.h>
 
 namespace tut
 {
@@ -50,6 +50,11 @@ namespace tut
 		{
 			factory_.destroyGeometry(mp_);
 		}
+
+    private:
+        // Declare type as noncopyable
+        test_multipoint_data(const test_multipoint_data& other);
+        test_multipoint_data& operator=(const test_multipoint_data& rhs);
 	};
 
 	typedef test_group<test_multipoint_data> group;
@@ -84,7 +89,7 @@ namespace tut
 	{
 		const size_t size0 = 0;
 		MultiPointAutoPtr copy(dynamic_cast<geos::geom::MultiPoint*>(empty_mp_->clone()));
-		ensure( copy.get() );
+		ensure( 0 != copy.get() );
 		
 		ensure( copy->isEmpty() );
 		ensure( copy->isSimple() );
