@@ -1,7 +1,9 @@
-// $Id: PolygonTest.cpp 2162 2008-08-18 16:30:56Z mloskot $
+// $Id: PolygonTest.cpp 2761 2009-12-03 18:56:25Z mloskot $
 // 
 // Test Suite for geos::geom::Polygon class.
 
+#include <tut.hpp>
+#include <utility.h>
 // geos
 #include <geos/geom/Polygon.h>
 #include <geos/geom/Coordinate.h>
@@ -17,10 +19,8 @@
 #include <geos/util/IllegalArgumentException.h>
 // std
 #include <cmath>
+#include <memory>
 #include <string>
-// tut
-#include <tut.h>
-#include <utility.h>
 
 namespace tut
 {
@@ -53,11 +53,16 @@ namespace tut
 			poly_ = static_cast<PolygonPtr>(geo);
 		}
 
-		~test_polygon_data() 
-		{
-			// FREE MEMORY
-			factory_.destroyGeometry(poly_);
-		}
+        ~test_polygon_data() 
+        {
+            // FREE MEMORY
+            factory_.destroyGeometry(poly_);
+        }
+
+    private:
+        // Declare type as noncopyable
+        test_polygon_data(const test_polygon_data& other);
+        test_polygon_data& operator=(const test_polygon_data& rhs);
     };
 
     typedef test_group<test_polygon_data> group;
@@ -345,7 +350,7 @@ namespace tut
     void object::test<24>()
 	{
 		ensure( poly_ != 0 );
-		ensure_not_equals( poly_->getLength(), 0.0 );
+		ensure( poly_->getLength() != 0.0 );
 
 		const double tolerance = 0.0001;
 		const double expected = 38.284271247461902;
@@ -359,7 +364,7 @@ namespace tut
     void object::test<25>()
 	{
 		ensure( poly_ != 0 );
-		ensure_not_equals( poly_->getArea(), 0.0 );
+		ensure( poly_->getArea() != 0.0 );
 	}
 	
 	// Test of getCoordinates() for non-empty Polygon
