@@ -1,10 +1,9 @@
-// $Id: DoubleBitsTest.cpp 1820 2006-09-06 16:54:23Z mloskot $
+// $Id: DoubleBitsTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::index::quadtree::DoubleBits class.
 
-// TUT
-#include <tut.h>
-// GEOS
+#include <tut.hpp>
+// geos
 #include <geos/index/quadtree/DoubleBits.h>
 
 using namespace geos::index::quadtree;
