@@ -1,14 +1,15 @@
-// $Id: SegmentPointComparatorTest.cpp 1820 2006-09-06 16:54:23Z mloskot $
+// $Id: SegmentPointComparatorTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::noding::SegmentPointComparator class.
 
-// TUT
-#include <tut.h>
-// GEOS
+#include <tut.hpp>
+// geos
 #include <geos/noding/SegmentNode.h>
+#include <geos/noding/SegmentString.h>
 #include <geos/geom/Coordinate.h>
 #include <geos/geom/CoordinateSequence.h>
 #include <geos/geom/CoordinateArraySequenceFactory.h>
+// std
 #include <memory>
 
 namespace tut
