@@ -1,7 +1,9 @@
-// $Id: TopologyPreservingSimplifierTest.cpp 2162 2008-08-18 16:30:56Z mloskot $
+// $Id: TopologyPreservingSimplifierTest.cpp 2344 2009-04-09 21:46:30Z mloskot $
 // 
 // Test Suite for geos::simplify::TopologyPreservingSimplifier
 
+#include <tut.hpp>
+#include <utility.h>
 // geos
 #include <geos/io/WKTReader.h>
 #include <geos/geom/PrecisionModel.h>
@@ -12,9 +14,6 @@
 // std
 #include <string>
 #include <memory>
-// tut
-#include <tut.h>
-#include <utility.h>
 
 namespace tut
 {
