@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: XMLTester.cpp 2131 2008-07-15 22:04:51Z mloskot $
+ * $Id: XMLTester.cpp 2760 2009-12-03 14:48:08Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -14,10 +14,15 @@
  *
  **********************************************************************/
 
+#ifdef _MSC_VER
+# if defined(GEOS_DEBUG_MSVC_USE_VLD) && !defined(GEOS_TEST_USE_STACKWALKER)
+#  include <vld.h>
+# else
 //#define _CRTDBG_MAP_ALLOC
 //#include <stdlib.h>
-#ifdef _MSC_VER
-#include <crtdbg.h>
+#  include <crtdbg.h>
+# endif
+#pragma warning(disable : 4127)
 #endif
 
 #include <geos/geom/Point.h>
@@ -30,6 +35,9 @@
 #include <geos/geom/PrecisionModel.h>
 #include <geos/geom/BinaryOp.h>
 #include <geos/operation/overlay/OverlayOp.h>
+#include <geos/operation/buffer/BufferBuilder.h>
+#include <geos/operation/buffer/BufferParameters.h>
+#include <geos/operation/buffer/BufferOp.h>
 #include <geos/util.h>
 //#include <geos/geomgraph.h>
 #include <geos/io/WKBReader.h>
@@ -40,10 +48,11 @@
 #include <geos/opPolygonize.h>
 #include <geos/opLinemerge.h>
 #include <geos/profiler.h>
-#include "markup/MarkupSTL.h"
 #include <geos/unload.h>
 #include <geos/opValid.h>
 #include "XMLTester.h"
+#include "BufferResultMatcher.h"
+#include "SingleSidedBufferResultMatcher.h"
 
 #include <cassert>
 #include <cctype>
@@ -52,20 +61,93 @@
 #include <functional>
 #include <iostream>
 #include <sstream>
-#include <string>
+#include <cstring>
 #include <memory>
 #include <functional>
 #include <stdexcept>
 #include <cmath>
+#include <stdexcept>
+#include <algorithm>
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && defined(GEOS_TEST_USE_STACKWALKER)
 #include <windows.h>
 #include "Stackwalker.h"
 #endif
 
+// Undefine this to use Geometry methods for overlay operations
+//
+// Using BinaryOp gives more successes though...
+//
+#define USE_BINARYOP 1
+
 using namespace geos;
 using namespace geos::operation::polygonize;
 using namespace geos::operation::linemerge;
+using std::runtime_error;
+
+namespace {
+
+// a utility function defining a very simple method to indent a line of text
+const char * getIndent( unsigned int numIndents )
+{
+    static const char * pINDENT = "                                      + ";
+    static const unsigned int LENGTH = strlen( pINDENT );
+
+    if ( numIndents > LENGTH ) numIndents = LENGTH;
+
+    return &pINDENT[ LENGTH-numIndents ];
+}
+
+
+
+void dump_to_stdout( const TiXmlNode * pParent, unsigned int indent = 0 )
+{
+    if ( !pParent ) return;
+
+    const TiXmlText *pText;
+    int t = pParent->Type();
+    printf( "%s", getIndent( indent));
+
+    switch ( t )
+    {
+    case TiXmlNode::DOCUMENT:
+        printf( "Document" );
+        break;
+
+    case TiXmlNode::ELEMENT:
+        printf( "Element \"%s\"", pParent->Value() );
+        break;
+
+    case TiXmlNode::COMMENT:
+        printf( "Comment: \"%s\"", pParent->Value());
+        break;
+
+    case TiXmlNode::UNKNOWN:
+        printf( "Unknown" );
+        break;
+
+    case TiXmlNode::TEXT:
+        pText = pParent->ToText();
+        printf( "Text: [%s]", pText->Value() );
+        break;
+
+    case TiXmlNode::DECLARATION:
+        printf( "Declaration" );
+        break;
+    default:
+        break;
+    }
+    printf( "\n" );
+
+    const TiXmlNode * pChild;
+
+    for ( pChild = pParent->FirstChild(); pChild != 0; pChild = pChild->NextSibling()) 
+    {
+        dump_to_stdout( pChild, indent+2 );
+    }
+}
+
+}
 
 void
 tolower(std::string& str)
@@ -92,6 +174,87 @@ normalize_filename(const std::string& str)
 	return newstring;
 }
 
+/* Could be an XMLTester class private but oh well.. */
+static int
+checkBufferSuccess(geom::Geometry const& gRes, geom::Geometry const& gRealRes, double dist)
+{
+	int success = 1;
+	do
+	{
+
+		if ( gRes.getGeometryTypeId() != gRealRes.getGeometryTypeId() )
+		{
+			std::cerr << "Expected result is of type "
+				<< gRes.getGeometryType()
+				<< "; obtained result is of type "
+				<< gRealRes.getGeometryType()
+				<< std::endl;
+			success=0;
+			break;
+		}
+
+		// Is a buffer always an area ?
+		if ( gRes.getDimension() != 2 )
+		{
+			std::cerr << "Don't know how to validate "
+				<< "result of buffer operation "
+				<< "when expected result is not an "
+				<< "areal type."
+				<< std::endl;
+		}
+		
+
+		geos::xmltester::BufferResultMatcher matcher;
+		if ( ! matcher.isBufferResultMatch(gRealRes,
+						   gRes,
+						   dist) )
+		{
+std::cerr << "BufferResultMatcher FAILED" << std::endl;
+			success=0;
+			break;
+		}
+
+	}
+	while (0);
+
+	return success;
+}
+
+static int
+checkSingleSidedBufferSuccess(geom::Geometry& gRes,
+		geom::Geometry& gRealRes, double dist)
+{
+	int success = 1;
+	do
+	{
+
+		if ( gRes.getGeometryTypeId() != gRealRes.getGeometryTypeId() )
+		{
+			std::cerr << "Expected result is of type "
+				<< gRes.getGeometryType()
+				<< "; obtained result is of type "
+				<< gRealRes.getGeometryType()
+				<< std::endl;
+			success=0;
+			break;
+		}
+
+		geos::xmltester::SingleSidedBufferResultMatcher matcher;
+		if ( ! matcher.isBufferResultMatch(gRealRes,
+						   gRes,
+						   dist) )
+		{
+std::cerr << "SingleSidedBufferResultMatcher FAILED" << std::endl;
+			success=0;
+			break;
+		}
+
+	}
+	while (0);
+
+	return success;
+}
+
 XMLTester::XMLTester()
 	:
 	gA(0),
@@ -219,22 +382,20 @@ XMLTester::run(const std::string &source)
 
 	caseCount=0;
 
-	if ( ! xml.Load(source.c_str()) )
+	if ( ! xml.LoadFile(source.c_str()) )
 	{
 		std::cerr << "Error loading " << source << std::endl;
 	}
 
-	xml.ResetPos();
-	xml.FindElem("run");
-	xml.FindChildElem("precisionModel");
-	parsePrecisionModel();
-	while (xml.FindChildElem("case")) {
-		try {
-			parseCase();
-		} catch (const std::exception& exc) {
-			std::cerr<<exc.what()<<std::endl;
-		}
-	}
+	//dump_to_stdout(&xml);
+
+	const TiXmlNode* node = xml.FirstChild("run");
+
+	if ( ! node )
+	  throw(runtime_error("Document has no childs"));
+
+	parseRun(node);
+
 }
 
 void 
@@ -252,17 +413,46 @@ XMLTester::resetCounters()
 	testFileCount=totalTestCount=failed=succeeded=0;
 }
 
+void
+XMLTester::parseRun(const TiXmlNode* node)
+{
+	assert(node);
+
+	//dump_to_stdout(node);
+
+	// Look for precisionModel element
+	const TiXmlElement* el = node->FirstChildElement("precisionModel");
+	if ( el ) parsePrecisionModel(el);
+
+	const TiXmlNode* casenode;
+	for ( casenode = node->FirstChild("case");
+	      casenode;
+	      casenode = casenode->NextSibling("case") )
+	{
+		try {
+			parseCase(casenode);
+		} catch (const std::exception& exc) {
+			std::cerr<<exc.what()<<std::endl;
+		}
+	}
+
+}
 
 void
-XMLTester::parsePrecisionModel()
+XMLTester::parsePrecisionModel(const TiXmlElement* el)
 {
 	using geos::geom::PrecisionModel;
 
-	/* This does not seem to work... */
-	std::string type=xml.GetChildAttrib("type");
-	std::string scaleStr=xml.GetChildAttrib("scale");
+	//dump_to_stdout(el);
 
-	if ( scaleStr == "" ) {
+	/* This does not seem to work... */
+	std::string type;
+	const char* typeStr = el->Attribute("type");
+	if ( typeStr ) type = typeStr;
+
+	const char* scaleStr = el->Attribute("scale");
+
+	if ( ! scaleStr ) {
 		if ( type == "FLOATING_SINGLE" )
 		{
 			pm.reset(new PrecisionModel(PrecisionModel::FLOATING_SINGLE));
@@ -272,15 +462,21 @@ XMLTester::parsePrecisionModel()
 			pm.reset(new PrecisionModel());
 		}
 	} else {
-		char* stopstring;
-		//string scaleStr=xml.GetChildAttrib("scale");
-		std::string offsetXStr=xml.GetChildAttrib("offsetx");
-		std::string offsetYStr=xml.GetChildAttrib("offsety");
 
-		double scale=std::strtod(scaleStr.c_str(),&stopstring);
-		double offsetX=std::strtod(offsetXStr.c_str(),&stopstring);
-		double offsetY=std::strtod(offsetYStr.c_str(),&stopstring);
-		pm.reset(new PrecisionModel(scale,offsetX,offsetY));
+		char* stopstring;
+
+		double scale = std::strtod(scaleStr, &stopstring);
+		double offsetX = 0;
+		double offsetY = 2;
+
+		if ( ! el->QueryDoubleAttribute("offsetx", &offsetX) )
+		{} // std::cerr << "No offsetx" << std::endl;
+
+		if ( ! el->QueryDoubleAttribute("offsety", &offsetY) )
+		{} // std::cerr << "No offsety" << std::endl;
+
+		// NOTE: PrecisionModel discards offsets anyway...
+		pm.reset(new PrecisionModel(scale, offsetX, offsetY));
 	}
 
 	if (verbose > 1)
@@ -323,6 +519,9 @@ XMLTester::testValid(const geom::Geometry* g, const std::string& label)
 geom::Geometry *
 XMLTester::parseGeometry(const std::string &in, const char* label)
 {
+	if ( ( ! wkbreader.get() ) || ( ! wktreader.get() ) )
+		throw(runtime_error("No precision model specified"));
+
 	std::stringstream is(in, std::ios_base::in);
 	char first_char;
 
@@ -376,8 +575,10 @@ XMLTester::trimBlanks(const std::string &in)
 }
 
 void
-XMLTester::parseCase()
+XMLTester::parseCase(const TiXmlNode* node)
 {
+	assert(node);
+
 	std::string geomAin;
 	std::string geomBin;
 	std::string thrownException;
@@ -385,21 +586,30 @@ XMLTester::parseCase()
 	gA=NULL;
 	gB=NULL;
 
-	xml.IntoElem();
-	xml.FindChildElem("desc");
-	curr_case_desc=trimBlanks(xml.GetChildData());
+
+	//dump_to_stdout(node);
+
+	curr_case_desc.clear();
+	const TiXmlNode* txt = node->FirstChild("desc");
+	if ( txt ) {
+		txt = txt->FirstChild();
+		if ( txt ) curr_case_desc = trimBlanks(txt->Value());
+	}
+
+	//std::cerr << "Desc: " << curr_case_desc << std::endl;
+
 
 	try {
-		xml.FindChildElem("a");
-		geomAin=xml.GetChildData();
-		geomAin=trimBlanks(geomAin);
-		gA=parseGeometry(geomAin, "Geometry A");
+		const TiXmlNode *el = node->FirstChild("a");
+		geomAin = el->FirstChild()->Value();
+		geomAin = trimBlanks(geomAin);
+		gA = parseGeometry(geomAin, "Geometry A");
 
-		if ( xml.FindChildElem("b") )
+		if ( 0 != (el = node->FirstChild("b")) )
 		{
-			geomBin=xml.GetChildData();
-			geomBin=trimBlanks(geomBin);
-			gB=parseGeometry(geomBin, "Geometry B");
+			geomBin = el->FirstChild()->Value();
+			geomBin = trimBlanks(geomBin);
+			gB = parseGeometry(geomBin, "Geometry B");
 		}
 	}
 	catch (const std::exception &e) {
@@ -409,23 +619,31 @@ XMLTester::parseCase()
 		thrownException = "Unknown exception";
 	}
 
+//std::cerr << "A: " << geomAin << std::endl;
+//std::cerr << "B: " << geomBin << std::endl;
+
+
 	if ( thrownException != "" )
 	{
 		std::cout << *curr_file <<":";
 		std::cout << " case" << caseCount << ":";
 		std::cout << " skipped ("<<thrownException<<")."<<std::endl;
-		xml.OutOfElem();
 		return;
 	}
 
 	++caseCount;
 	testCount=0;
-	while(xml.FindChildElem("test")) {
-		parseTest();
+
+	const TiXmlNode* testnode;
+	for ( testnode = node->FirstChild("test");
+	      testnode;
+	      testnode = testnode->NextSibling("test") )
+	{
+		parseTest(testnode);
 	}
+
 	totalTestCount+=testCount;
 		
-	xml.OutOfElem();
 	delete gA;
 	delete gB;
 }
@@ -455,7 +673,7 @@ XMLTester::printGeom(geom::Geometry *g)
 }
 
 void
-XMLTester::parseTest()
+XMLTester::parseTest(const TiXmlNode* node)
 {
 	using namespace operation::overlay;
 
@@ -466,19 +684,43 @@ XMLTester::parseTest()
 	std::string opArg1;
 	std::string opArg2;
 	std::string opArg3;
+	std::string opArg4;
 	std::string opRes;
-	//string opSig;
 
 	++testCount;
 
-	xml.IntoElem();
-	xml.FindChildElem("op");
-	opName=xml.GetChildAttrib("name");
-	opArg1=xml.GetChildAttrib("arg1");
-	opArg2=xml.GetChildAttrib("arg2");
-	opArg3=xml.GetChildAttrib("arg3");
-	//opSig=xml.GetChildAttrib("arg3");
-	opRes=xml.GetChildData();
+	const TiXmlNode* opnode = node->FirstChild("op");
+	if ( ! opnode ) throw(runtime_error("case has no op"));
+
+	//dump_to_stdout(opnode);
+
+	const TiXmlElement* opel = opnode->ToElement();
+
+	const char* tmp = opel->Attribute("name");
+	if ( tmp ) opName = tmp;
+
+	tmp = opel->Attribute("arg1");
+	if ( tmp ) opArg1 = tmp;
+
+	tmp = opel->Attribute("arg2");
+	if ( tmp ) opArg2 = tmp;
+
+	tmp = opel->Attribute("arg3");
+	if ( tmp ) opArg3 = tmp;
+
+	tmp = opel->Attribute("arg4");
+	if ( tmp ) opArg4 = tmp;
+
+	const TiXmlNode* resnode = opnode->FirstChild();
+	if ( ! resnode )
+	{
+		std::stringstream tmp;
+		tmp << "op of test " << testCount
+		    << " of case " << caseCount
+		    << " has no expected result child";
+		throw(runtime_error(tmp.str()));
+	}
+	opRes = resnode->Value();
 
 	// trim blanks
 	opRes=trimBlanks(opRes);
@@ -496,6 +738,10 @@ XMLTester::parseTest()
 		if ( opSig != "" ) opSig += ", ";
 		opSig += opArg3;
 	}
+	if ( opArg4 != "" ) {
+		if ( opSig != "" ) opSig += ", ";
+		opSig += opArg4;
+	}
 
 	opSignature = opName + "(" + opSig + ")";
 
@@ -541,8 +787,12 @@ XMLTester::parseTest()
 			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
 			gRes->normalize();
 
-			//GeomAutoPtr gRealRes(gA->intersection(gB));
+#ifndef USE_BINARYOP
+			GeomAutoPtr gRealRes(gA->intersection(gB));
+#else
 			GeomAutoPtr gRealRes = BinaryOp(gA, gB, overlayOp(OverlayOp::opINTERSECTION));
+#endif
+
 			gRealRes->normalize();
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
@@ -559,8 +809,12 @@ XMLTester::parseTest()
 			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
 			gRes->normalize();
 
-			//GeomAutoPtr gRealRes(gA->Union(gB));
+#ifndef USE_BINARYOP
+			GeomAutoPtr gRealRes(gA->Union(gB));
+#else
 			GeomAutoPtr gRealRes = BinaryOp(gA, gB, overlayOp(OverlayOp::opUNION));
+#endif
+
 			gRealRes->normalize();
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
@@ -577,8 +831,11 @@ XMLTester::parseTest()
 			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
 			gRes->normalize();
 
-			//GeomAutoPtr gRealRes(gA->difference(gB));
+#ifndef USE_BINARYOP
+			GeomAutoPtr gRealRes(gA->difference(gB));
+#else
 			GeomAutoPtr gRealRes = BinaryOp(gA, gB, overlayOp(OverlayOp::opDIFFERENCE));
+#endif
 			
 			gRealRes->normalize();
 
@@ -596,8 +853,12 @@ XMLTester::parseTest()
 			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
 			gRes->normalize();
 
-			//GeomAutoPtr gRealRes(gA->symDifference(gB));
+#ifndef USE_BINARYOP
+			GeomAutoPtr gRealRes(gA->symDifference(gB));
+#else
 			GeomAutoPtr gRealRes = BinaryOp(gA, gB, overlayOp(OverlayOp::opSYMDIFFERENCE));
+#endif
+
 			gRealRes->normalize();
 
 			if (gRes->compareTo(gRealRes.get())==0) success=1;
@@ -611,7 +872,54 @@ XMLTester::parseTest()
 
 		else if (opName=="intersects")
 		{
-			if (gA->intersects(gB)) actual_result="true";
+			geom::Geometry *g1 = opArg1 == "B" ? gB : gA;
+			geom::Geometry *g2 = opArg2 == "B" ? gB : gA;
+
+			if (g1->intersects(g2)) actual_result="true";
+			else actual_result="false";
+			
+			if (actual_result==opRes) success=1;
+		}
+
+		else if (opName=="contains")
+		{
+			geom::Geometry *g1 = opArg1 == "B" ? gB : gA;
+			geom::Geometry *g2 = opArg2 == "B" ? gB : gA;
+
+			if (g1->contains(g2)) actual_result="true";
+			else actual_result="false";
+			
+			if (actual_result==opRes) success=1;
+		}
+
+		else if (opName=="within")
+		{
+			geom::Geometry *g1 = opArg1 == "B" ? gB : gA;
+			geom::Geometry *g2 = opArg2 == "B" ? gB : gA;
+
+			if (g1->within(g2)) actual_result="true";
+			else actual_result="false";
+			
+			if (actual_result==opRes) success=1;
+		}
+
+		else if (opName=="covers")
+		{
+			geom::Geometry *g1 = opArg1 == "B" ? gB : gA;
+			geom::Geometry *g2 = opArg2 == "B" ? gB : gA;
+
+			if (g1->covers(g2)) actual_result="true";
+			else actual_result="false";
+			
+			if (actual_result==opRes) success=1;
+		}
+
+		else if (opName=="coveredby")
+		{
+			geom::Geometry *g1 = opArg1 == "B" ? gB : gA;
+			geom::Geometry *g2 = opArg2 == "B" ? gB : gA;
+
+			if (g1->coveredBy(g2)) actual_result="true";
 			else actual_result="false";
 			
 			if (actual_result==opRes) success=1;
@@ -694,6 +1002,8 @@ XMLTester::parseTest()
 
 		else if (opName=="buffer")
 		{
+			using namespace operation::buffer;
+
 			geom::Geometry *gT=gA;
 			if ( ( opArg1 == "B" || opArg1 == "b" ) && gB ) gT=gB;
 
@@ -703,79 +1013,22 @@ XMLTester::parseTest()
 			profile.start();
 
 			GeomAutoPtr gRealRes;
-			double dist = std::atof(opArg3.c_str());
+			double dist = std::atof(opArg2.c_str());
 
-			if ( opArg2 != "" ) {
-				gRealRes.reset(gT->buffer(dist, std::atoi(opArg2.c_str())));
-			} else {
-				gRealRes.reset(gT->buffer(dist));
+			BufferParameters params;
+			if ( opArg3 != "" ) {
+		params.setQuadrantSegments(std::atoi(opArg3.c_str()));
 			}
+			
+
+			BufferOp op(gT, params);
+			gRealRes.reset(op.getResultGeometry(dist));
+
 			profile.stop();
 			gRealRes->normalize();
 
-			// Assume a success and check for obvious failures
-			success=1;
-			do
-			{
-				// TODO: Is a buffer always an area ?
-				// 	 we might check geometry type..
-
-				if ( gRes->getGeometryTypeId() != gRealRes->getGeometryTypeId() )
-				{
-					std::cerr << "Expected result is of type "
-					        << gRes->getGeometryType()
-						<< "; obtained result is of type "
-						<< gRealRes->getGeometryType()
-						<< std::endl;
-					success=0;
-					break;
-				}
-
-				if ( gRes->isEmpty() && gRealRes->isEmpty() )
-				{
-					// Success !
-					break;
-				}
-
-				if ( gRes->getDimension() != 2 )
-				{
-					std::cerr << "Don't know how to validate "
-						<< "result of buffer operation "
-						<< "when expected result is not an "
-						<< "areal type."
-						<< std::endl;
-				}
-				
-				double expectedArea = gRes->getArea();
-
-				/// Allow area difference being at most
-				/// 1/1000 of the area of the expected result.
-				double areatol = expectedArea / 1e3;
-
-				GeomAutoPtr gDiff = BinaryOp(gRes.get(), gRealRes.get(),
-					overlayOp(OverlayOp::opDIFFERENCE));
-
-				double areaDiff = gDiff->getArea();
-				if ( areaDiff > areatol )
-				{
-					std::cerr << "Area of difference between "
-						<< "obtained and expected: "
-						<< areaDiff << " - Tolerated diff: "
-						<< areatol << std::endl;
-					success=0;
-					break;
-				}
-				else
-				{
-					std::cerr << "Area of difference between "
-						<< "obtained and expected: "
-						<< areaDiff << " - Tolerated diff: "
-						<< areatol << " (SUCCESS!)"
-						<< std::endl;
-				}
-
-			}
-			while (0);
+			// Validate the buffer operation
+			success = checkBufferSuccess(*gRes, *gRealRes, dist);
 
 			if ( testValidOutput ) testValid(gRes.get(), "result");
 
@@ -783,6 +1036,88 @@ XMLTester::parseTest()
 			expected_result=printGeom(gRes.get());
 		}
 
+		else if (opName=="buffersinglesided")
+		{
+			using namespace operation::buffer;
+
+			geom::Geometry *gT=gA;
+			if ( ( opArg1 == "B" || opArg1 == "b" ) && gB ) gT=gB;
+
+			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
+			gRes->normalize();
+
+			profile.start();
+
+			GeomAutoPtr gRealRes;
+			double dist = std::atof(opArg2.c_str());
+				
+			BufferParameters params ;
+			params.setJoinStyle( BufferParameters::JOIN_ROUND ) ;
+			if ( opArg3 != "" ) {
+		params.setQuadrantSegments( std::atoi(opArg3.c_str()));
+			}
+			
+			bool leftSide = true ;
+			if ( opArg4 == "right" )
+			{
+				leftSide = false ;
+			}
+
+			BufferBuilder bufBuilder( params ) ;
+			gRealRes.reset( bufBuilder.bufferLineSingleSided(
+			                             gT, dist, leftSide ) ) ;
+
+			profile.stop();
+			gRealRes->normalize();
+
+			// Validate the single sided buffer operation
+			success = checkSingleSidedBufferSuccess(*gRes,
+					*gRealRes, dist);
+
+			if ( testValidOutput ) testValid(gRes.get(), "result");
+
+			actual_result=printGeom(gRealRes.get());
+			expected_result=printGeom(gRes.get());
+		}
+
+		else if (opName=="buffermitredjoin")
+		{
+			using namespace operation::buffer;
+
+			geom::Geometry *gT=gA;
+			if ( ( opArg1 == "B" || opArg1 == "b" ) && gB ) gT=gB;
+
+			GeomAutoPtr gRes(parseGeometry(opRes, "expected"));
+			gRes->normalize();
+
+			profile.start();
+
+			GeomAutoPtr gRealRes;
+			double dist = std::atof(opArg2.c_str());
+
+			BufferParameters params;
+			params.setJoinStyle(BufferParameters::JOIN_MITRE);
+
+			if ( opArg3 != "" ) {
+		params.setQuadrantSegments(std::atoi(opArg3.c_str()));
+			}
+
+			BufferOp op(gT, params);
+			gRealRes.reset(op.getResultGeometry(dist));
+
+			profile.stop();
+			gRealRes->normalize();
+
+			// Validate the buffer operation
+			success = checkBufferSuccess(*gRes, *gRealRes, dist);
+
+			if ( testValidOutput ) testValid(gRes.get(), "result");
+
+			actual_result=printGeom(gRealRes.get());
+			expected_result=printGeom(gRes.get());
+		}
+
+
 		else if (opName=="getinteriorpoint")
 		{
 			geom::Geometry *gT=gA;
@@ -806,7 +1141,7 @@ XMLTester::parseTest()
 
 		else if (opName=="iswithindistance")
 		{
-			float dist=std::atof(opArg3.c_str());
+			double dist=std::atof(opArg3.c_str());
 			if (gA->isWithinDistance(gB, dist)) {
 				actual_result="true";
 			} else {
@@ -886,8 +1221,12 @@ XMLTester::parseTest()
 			{
 		std::cerr << "Running intersection for areatest" << std::endl;
 			}
+#ifndef USE_BINARYOP
+			GeomAutoPtr gI(gA->intersection(gB));
+#else
 			GeomAutoPtr gI = BinaryOp(gA, gB,
 					overlayOp(OverlayOp::opINTERSECTION));
+#endif
 
 			if ( testValidOutput )
 			{
@@ -898,15 +1237,25 @@ XMLTester::parseTest()
 			{
 		std::cerr << "Running difference(A,B) for areatest" << std::endl;
 			}
+
+#ifndef USE_BINARYOP
+			GeomAutoPtr gDab(gA->difference(gB));
+#else
 			GeomAutoPtr gDab = BinaryOp(gA, gB,
 					overlayOp(OverlayOp::opDIFFERENCE));
+#endif
 
 			if ( verbose > 1 )
 			{
 		std::cerr << "Running difference(B,A) for areatest" << std::endl;
 			}
+
+#ifndef USE_BINARYOP
+			GeomAutoPtr gDba(gB->difference(gA));
+#else
 			GeomAutoPtr gDba = BinaryOp(gB, gA,
 					overlayOp(OverlayOp::opDIFFERENCE));
+#endif
 
 			if ( testValidOutput )
 			{
@@ -917,15 +1266,25 @@ XMLTester::parseTest()
 			{
 		std::cerr << "Running symdifference for areatest" << std::endl;
 			}
+
+#ifndef USE_BINARYOP
+			GeomAutoPtr gSD(gA->symDifference(gB));
+#else
 			GeomAutoPtr gSD = BinaryOp(gA, gB,
 					overlayOp(OverlayOp::opSYMDIFFERENCE));
+#endif
 
 			if ( verbose > 1 )
 			{
 		std::cerr << "Running union for areatest" << std::endl;
 			}
+
+#ifndef USE_BINARYOP
+			GeomAutoPtr gU(gA->Union(gB));
+#else
 			GeomAutoPtr gU = BinaryOp(gA, gB,
 					overlayOp(OverlayOp::opUNION));
+#endif
 
 			double areaA = gA->getArea();
 			double areaB = gB->getArea();
@@ -1012,14 +1371,13 @@ XMLTester::parseTest()
 
 	if ((!success && verbose) || verbose > 1)
 	{
-		printTest(success, expected_result, actual_result);
+		printTest(!!success, expected_result, actual_result);
 	}
 
 	if (test_predicates && gB && gA) {
 		runPredicates(gA, gB);
 	}
 
-	xml.OutOfElem();
 }
 
 void
@@ -1061,7 +1419,7 @@ main(int argC, char* argV[])
 	int verbose=0;
 	bool sql_output=false;
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && defined(GEOS_TEST_USE_STACKWALKER)
 	InitAllocCheck();
 	{
 #endif
@@ -1114,7 +1472,7 @@ main(int argC, char* argV[])
 
 	return tester.getFailuresCount();
 
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && defined(GEOS_TEST_USE_STACKWALKER)
 	}
 	DeInitAllocCheck();
 #endif
